#include "Applet.h"

Applet::Applet(QWidget *parent)
    : QSystemTrayIcon(parent),
      daemon( "com.ubudsl.Daemon", "/com/ubudsl/Daemon" , "com.ubudsl.Daemon", QDBusConnection::systemBus() ),
      m_daemonOk(false),
      m_menu(new QMenu()),
      iconConnected(":/images/connected.png"),
      iconConnecting(":/images/connecting.png"),
      iconDisconnected(":/images/disconnected.png"),
      iconHide(":/images/hide.png"),
      iconNoDaemon(":/images/nodaemon.png"),
      iconNoModem(":/images/nomodem.png"),
      iconShow(":/images/show.png"),
      m_newModemMsgBox(new QMessageBox(0)),
      m_connectionTimer(new QTimer()),
      m_timer(new QTimer()),
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
      sndConnected(":sounds/connected.wav"),
      sndDisconnected(":sounds/disconnected.wav"),
      sndError(":sounds/error.wav"),
      sndPlugged(":sounds/modem_plugged.wav"),
      sndUnplugged(":sounds/modem_unplugged.wav"),
#endif
      m_monitor(new Monitor()),
      m_generateLog(new GenerateLog(this)),
      m_settingsDialog(new Settings())
{
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
        qDebug() << "Initialising phonon objects...";
        audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
        mediaObject = new Phonon::MediaObject(this);
        qDebug() << "Initialising phonon objects finished.";
#endif
	
	m_actionHide = m_menu->addAction(iconShow, tr("&Show"));
	m_actionConnect = m_menu->addAction(iconConnected, tr("&Connect"));
	m_actionReconnect = m_menu->addAction(iconConnected, tr("&Reconnect"));
	m_separator1 = m_menu->addSeparator();
	m_actionGenerateLog = m_menu->addAction(QIcon(":/images/log.png"), tr("&Generate log"));
	m_actionRestartModem = m_menu->addAction(QIcon(":/images/restart.png"), tr("&Restart modem"));
	m_actionSettings = m_menu->addAction(QIcon(":/images/settings.png"), tr("&Settings"));
	m_separator2 = m_menu->addSeparator();
	m_actionQuit = m_menu->addAction(QIcon(":/images/quit.png"), tr("&Quit"));
	
	m_actionConnect->setEnabled(false);
	m_monitor->setConnectEnabled(false);
	
	connect(m_monitor, SIGNAL(showHide()), this, SLOT(showHideMonitor()));
	connect(m_monitor, SIGNAL(enableConnectButton(bool)), m_actionConnect, SLOT(setEnabled(bool)));
	connect(m_actionConnect, SIGNAL(triggered()), m_monitor, SLOT(connectDisconnect()));
	connect(m_actionReconnect, SIGNAL(triggered()), m_monitor, SLOT(reconnect()));
	connect(m_actionHide, SIGNAL(triggered()), this, SLOT(showHideMonitor()));
	connect(m_actionQuit, SIGNAL(triggered()), this, SIGNAL(quit()));
	connect(m_actionGenerateLog, SIGNAL(triggered()), m_generateLog, SLOT(start()));
	connect(m_actionSettings, SIGNAL(triggered()), this, SLOT(showSettings()));
	connect(m_actionRestartModem, SIGNAL(triggered()), this, SLOT(restartModem()));
	connect(m_generateLog, SIGNAL(started()), this, SLOT(disableGenerateLog()));
	connect(m_generateLog, SIGNAL(finished()), this, SLOT(enableGenerateLog()));
	connect(m_timer, SIGNAL(timeout()), this, SLOT(checkDaemonConnection()));
	connect(m_connectionTimer, SIGNAL(timeout()), this, SLOT(connecting()));
	connect(m_settingsDialog->ui.buttonSave, SIGNAL(clicked()), this, SLOT(applySettings()));
	connect( this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
			 this, SLOT(clicked( QSystemTrayIcon::ActivationReason )) );
	
	setContextMenu( m_menu );
	
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "connectionEstablished",
			                             this, SLOT(connectionEstablished()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "connectionTerminated",
			                             this, SLOT(connectionTerminated()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "establishingConnection",
			                             this, SLOT(establishingConnection()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "restartModemStarted",
			                             this, SLOT(restartModemStarted()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "restartModemFinished",
			                             this, SLOT(restartModemFinished()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "connectionTimeout",
			                             this, SLOT(connectionTimeout()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "settingOpenDNS",
			                             this, SLOT(settingOpenDNS(const bool)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "incorrectDNS",
			                             this, SLOT(incorrectDNS()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "modemUnplugged",
			                             this, SLOT(modemUnplugged()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "newModemUnplugged",
			                             this, SLOT(newModemUnplugged()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "modemPlugged",
			                             this, SLOT(modemPlugged(const QString &)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "newModemPlugged",
			                             this, SLOT(newModemPlugged(const QString &)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "accountDataChanged",
			                             this, SLOT(accountDataChanged(const bool)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "compilationFinished",
			                             this, SLOT(compilationFinished(const bool)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "compilingDriver",
			                             this, SLOT(compilingDriver()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "connectionTimeout",
			                             this, SLOT(playErrorSound()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "synchronizing",
			                             this, SLOT(synchronizing()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "synchronizationFailed",
			                             this, SLOT(synchronizationFailed()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "invalidAccountData",
			                             this, SLOT(invalidAccountData()));
	
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
	Phonon::createPath(mediaObject, audioOutput);
#endif
	
	m_newModemMsgBox->setAttribute(Qt::WA_QuitOnClose, false);
	m_newModemMsgBox->setIcon(QMessageBox::Question);
	m_newModemMsgBox->addButton(tr("&No"), QMessageBox::NoRole);
	m_btnYes = m_newModemMsgBox->addButton(tr("&Yes"), QMessageBox::YesRole);
	
	QRect d(QApplication::desktop()->screenGeometry());
	m_monitor->move( (d.width() - m_monitor->width()) / 2, (d.height() - m_monitor->height()) / 2 );

	QDBusReply<int> state = daemon.call("getConnectionState");
	
	if ( state.isValid() )
	{
		QDBusReply<bool> modemPlugged = daemon.call("getIsModemPlugged");
 
		if (modemPlugged.isValid() && !modemPlugged.value())
			setIcon(iconNoModem);
		else if ( state == ConnectionState::Connected )
			setIcon(iconConnected);
		else if ( state == ConnectionState::Connecting )
			setIcon(iconConnecting);
		else
			setIcon(iconDisconnected);


                QDBusReply<bool> compilingDrv = daemon.call("getIsModemPlugged");

		if (compilingDrv.isValid() && compilingDrv.value())
			compilingDriver();   
	}
	else
		setIcon(iconNoDaemon);

	QTimer::singleShot(500, this, SLOT(checkState()));

}

Applet::~Applet()
{

}

void Applet::showHideMonitor()
{
	if ( m_monitor->isHidden() )
	{
		m_actionHide->setText(tr("&Hide"));
		m_actionHide->setIcon(iconHide);
		m_monitor->show();
	}
	else
	{
		m_actionHide->setText(tr("&Show"));
		m_actionHide->setIcon(iconShow);
		m_monitor->hide();
	}
}

void Applet::clicked( QSystemTrayIcon::ActivationReason reason )
{
	if ( reason == QSystemTrayIcon::Trigger )
		showHideMonitor();
}

void Applet::showMessage( Message msg )
{
	QString title;
	QString text;
	
	switch (msg)
	{
		case Connecting:
			title = tr("Establishing connection");
			text  = tr("Connection is being established...");
			break;
		case ConnectionTimeout:
			title = tr("Connection timeout");
			text  = tr("Could not establish connection - provider does not respond.");
			break;
		case RestartingModem:
			title = tr("Restarting modem");
			text  = tr("Modem is being restarted.\nThis may take few seconds...");
			break;
		case ConnectionTerminated:
			title = tr("Connection terminated");
			text  = tr("Connection has been terminated...");
			break;
		case NoDaemon:
			title = tr("Daemon error");
			text  = tr("Probably the UbuDSL Daemon is not running. Please "
					   "run it if you want to use this applet.");
			break;
		case IncorrectDNS:
			title = tr("Incorrect DNS servers");
			text  = tr("UbuDSL has detected damaged DNS servers "
					   "configuration. Trying to set OpenDNS as "
					   "the default DNS servers...");
			break;
		case OpenDNS_Success:
			title = tr("OpenDNS set");
			text  = tr("UbuDSL has successfully set OpenDNS "
					   "as the default DNS servers...");
			break;
		case OpenDNS_Failed:
			title = tr("Setting OpenDNS failed");
			text  = tr("UbuDSL could not set OpenDNS as the "
					   "default DNS servers...");
			break;
		case Synchronizing:
			title = tr("Waiting for synchronization");
			text  = tr("Please wait while modem is now "
					   "synchronizing...");
			break;
		case SynchronizationFailed:
			title = tr("Synchronization failed");
			text  = tr("Connection could not be established "
					   "because modem failed to synchronize...");
			break;
		case CompilingDriver:
			title = tr("Compiling driver");
			text  = tr("Please wait while the modem's driver is "
					   "being compiled...");
			break;
		case CompilationFailed:
			title = tr("Compilation failed!");
			text  = tr("Compilation has failed and most probably you will not "
					   "be able to establish connection. Please generate and send "
					   "us the log - we will try to help you out.");
			break;
		case CompilationSucceed:
			title = tr("Compilation succeed");
			text  = tr("Compilation has been finished successfully. "
					   "You should now be able to establish connection...");
			break;
	}
			
	if ( !title.isEmpty() && !text.isEmpty() )
		QSystemTrayIcon::showMessage( title, text );
}

void Applet::enableGenerateLog()
{
	m_actionGenerateLog->setEnabled(true);
}

void Applet::disableGenerateLog()
{
	m_actionGenerateLog->setEnabled(false);
}

void Applet::establishingConnection()
{
	m_connectionTimer->start(500);
	setIcon(iconConnecting);
	showMessage( Connecting );
	setToolTip(tr("Establishing connection..."));
}

void Applet::connectionEstablished()
{
	m_connectionTimer->stop();
	m_actionConnect->setIcon(iconDisconnected);
	
	setIcon(iconConnected);
	
	m_actionConnect->setText(tr("&Disconnect"));
	m_monitor->setConnectButtonText(m_actionConnect->text().toLower());
	
	QDBusReply<QString> interface = daemon.call("getInterface");
	QDBusReply<QString> ip = daemon.call("getIP");
	
	if ( interface.isValid() && ip.isValid() )
	{
		QString title = tr("Connection established");
		QString text  = tr("Interface:\t%1\nIP Address:\t%2");
		
		if ( QLocale::system().country() == QLocale::Greece )
			text.insert( text.indexOf('\t'), '\t' );
		
		QSystemTrayIcon::showMessage( title, text.arg(interface.value(), ip.value()) );
		setToolTip(QString(title + "\n" + text).arg( interface.value(), ip.value()));
	}
	
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
	mediaObject->setCurrentSource(sndConnected);
	mediaObject->play();
#endif
}

void Applet::connectionTerminated( bool justRun )
{
	m_connectionTimer->stop();
	m_actionConnect->setIcon(iconConnected);
	
	setIcon(iconDisconnected);
	
	m_actionConnect->setText(tr("&Connect"));
	m_monitor->setConnectButtonText(m_actionConnect->text().toLower());
	setToolTip( tr("Disconnected") );
	
	if ( !justRun )
		showMessage( ConnectionTerminated );
	
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
	mediaObject->setCurrentSource(sndDisconnected);
	mediaObject->play();
#endif
}

void Applet::restartModem()
{
	daemon.call("restartModem");
}

void Applet::checkState(bool firstTime)
{
	QDBusReply<int> state = daemon.call("getConnectionState");
	
	if ( state.isValid() )
	{
		QDBusReply<bool> modemPlugged = daemon.call("getIsModemPlugged");
		
		m_actionConnect->setEnabled(modemPlugged.isValid() && modemPlugged.value());
		m_monitor->setConnectEnabled(modemPlugged.isValid() && modemPlugged.value());
		
		if ( modemPlugged.isValid() && !modemPlugged.value() )
		{
			setIcon(iconNoModem);
			setToolTip(tr("Modem is unplugged..."));
			
			QString title = tr("Modem unplugged");
			QString text  = tr("Modem is unplugged...");
			
			QSystemTrayIcon::showMessage(title, text, QSystemTrayIcon::Warning);
		}
		else if ( state == ConnectionState::Connected )
			connectionEstablished();
		else if ( state == ConnectionState::Connecting )
			establishingConnection();
		else if ( state == ConnectionState::Disconnected )
			connectionTerminated(true);
	}
	else if ( !daemon.isValid() )
	{
		showMessage(NoDaemon);
		m_actionConnect->setEnabled(false);
		m_monitor->setConnectEnabled(false);
		setToolTip(tr("Failed to connect to the daemon!"));
	}
	
	if (firstTime)
		m_timer->start(2000);
}

void Applet::showSettings()
{
	if ( m_settingsDialog->isVisible() )
		return;
	
	m_settingsDialog->ui.buttonSave->setDisabled(true);
	
	QDBusReply<QString> strReply;
	QDBusReply<bool>    boolReply;
	QString             dataError(tr("data error"));
	
	strReply = daemon.call("getService");
	if (!strReply.isValid() || strReply.value() == "error")
		m_settingsDialog->ui.labelName->setText(dataError);
	else
		m_settingsDialog->ui.labelName->setText(strReply.value());
	
	strReply = daemon.call("getProtocol");
	if (!strReply.isValid() || strReply.value() == "error")
		m_settingsDialog->ui.labelProtocol->setText(dataError);
	else
		m_settingsDialog->ui.labelProtocol->setText(strReply.value());
	
	strReply = daemon.call("getVPI");
	if (!strReply.isValid() || strReply.value() == "error")
		m_settingsDialog->ui.labelVPI->setText(dataError);
	else
		m_settingsDialog->ui.labelVPI->setText(strReply.value());
	
	strReply = daemon.call("getVCI");
	if (!strReply.isValid() || strReply.value() == "error")
		m_settingsDialog->ui.labelVCI->setText(dataError);
	else
		m_settingsDialog->ui.labelVCI->setText(strReply.value());
	
	boolReply = daemon.call("getConnectAtBoot");
	m_settingsDialog->ui.cbConnectAtBoot->setEnabled(boolReply.isValid());
	if (boolReply.isValid())
		m_settingsDialog->ui.cbConnectAtBoot->setChecked(boolReply.value());
	
	boolReply = daemon.call("getConnectAtModemPlug");
	m_settingsDialog->ui.cbConnectAtModemPlug->setEnabled(boolReply.isValid());
	if (boolReply.isValid())
		m_settingsDialog->ui.cbConnectAtModemPlug->setChecked(boolReply.value());
	
	
	QDBusReply<QStringList> listReply = daemon.call("getLoginAndPassword");
	if (!listReply.isValid() || listReply.value().count() != 2)
	{
		m_settingsDialog->ui.leLogin->setText(dataError);
		m_settingsDialog->ui.lePassword->setText(dataError);
		m_settingsDialog->ui.leLogin->setEnabled(false);
		m_settingsDialog->ui.lePassword->setEnabled(false);
	}
	else
	{
		m_settingsDialog->ui.leLogin->setText(listReply.value().at(0));
		m_settingsDialog->ui.lePassword->setText(listReply.value().at(1));
		m_settingsDialog->ui.leLogin->setEnabled(true);
		m_settingsDialog->ui.lePassword->setEnabled(true);
	}
	
	QDBusReply<int> intReply = daemon.call("getIdlenessTime");
	m_settingsDialog->ui.cbIdle->setEnabled(intReply.isValid());
	if (intReply.isValid())
	{
		if (intReply.value() > 0 && intReply.value() <= 360)
		{
			m_settingsDialog->ui.cbIdle->setChecked(true);
			m_settingsDialog->ui.spinIdle->setValue(intReply.value());
		}
		else
		{
			m_settingsDialog->ui.cbIdle->setChecked(false);
		}
	}
	
	m_settingsDialog->show();
}

void Applet::applySettings()
{
	if ( m_settingsDialog->getAccountDataChanged() )
	{
		daemon.call("setAccountData", m_settingsDialog->ui.leLogin->text(), m_settingsDialog->ui.lePassword->text());
	}
	
	if ( m_settingsDialog->getConnectAtBootChanged() )
	{
		daemon.call("setConnectAtBoot", m_settingsDialog->ui.cbConnectAtBoot->isChecked());
	}
	
	if ( m_settingsDialog->getConnectAtModemPlugChanged() )
	{
		daemon.call("setConnectAtModemPlug", m_settingsDialog->ui.cbConnectAtModemPlug->isChecked());
	}
	
	if ( m_settingsDialog->getIdleTimeChanged() )
	{
		daemon.call("setIdlenessTime", m_settingsDialog->ui.spinIdle->value());
	}
	
	if ( m_settingsDialog->getIdleChanged() && !m_settingsDialog->ui.cbIdle->isChecked() )
	{
		daemon.call("setIdlenessTime", 0);
	}
	
	m_settingsDialog->getData();
}

void Applet::modemUnplugged()
{
	setIcon(iconNoModem);
	setToolTip("Modem is unplugged...");
	
	m_actionConnect->setEnabled(false);
	m_monitor->setConnectEnabled(false);
	
	QString title = tr("Modem unplugged");
	QString text  = tr("Modem has been unplugged...");
	
	QSystemTrayIcon::showMessage(title, text, QSystemTrayIcon::Warning);
}

void Applet::newModemPlugged( const QString &name )
{
	QString title = tr("New modem plugged in");
	QString text  = tr("%1 has been plugged in.");
	QString textL = tr("%1 has been plugged in. Do you want to set it as the default modem?");
	
	QSystemTrayIcon::showMessage(title, text.arg(name), QSystemTrayIcon::Information);
	
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
	mediaObject->setCurrentSource(sndPlugged);
#endif

	m_newModemMsgBox->setWindowTitle(title);
	m_newModemMsgBox->setText(textL.arg(name));
	m_newModemMsgBox->exec();
	
	if ( m_newModemMsgBox->clickedButton() == m_btnYes )
	{
		daemon.call("useNewModem");
	}
}

void Applet::modemPlugged( const QString &name )
{
	QString title = tr("Modem plugged");
	QString text  = (name.isEmpty() ? tr("Modem has been plugged in...") : tr("%1 has been plugged in..."));
	
	QSystemTrayIcon::showMessage(title, text.arg(name));
	m_actionConnect->setEnabled(true);//TODO: remove after synchro detection is implemented
	m_monitor->setConnectEnabled(true);
	setToolTip(tr("Disconnected"));
	setIcon(iconDisconnected);
	
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
	mediaObject->setCurrentSource(sndPlugged); //TODO: replace with proper sound
	mediaObject->play();
#endif
}


void Applet::incorrectDNS()
{
	showMessage(IncorrectDNS);
}

void Applet::settingOpenDNS( const bool success )
{
	if ( success )
		showMessage(OpenDNS_Success);
	else
		showMessage(OpenDNS_Failed);
}

void Applet::newModemUnplugged()
{
	QString title = tr("New modem unplugged");
	QString text  = tr("New modem has been unplugged...");
	
	QSystemTrayIcon::showMessage(title, text, QSystemTrayIcon::Warning);
	
	m_newModemMsgBox->reject();
	
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
	mediaObject->setCurrentSource(sndUnplugged);
	mediaObject->play();
#endif
}

void Applet::accountDataChanged(const bool success)
{
	QString title = (success ? tr("Account data changed") : tr("Failed to change data"));
	QString text;
	
	if (success)
	{
		text = tr("Account data changed successfully...");
	}
	else
	{
		text = tr("An error has occured while changing the account data...");
	}

	QSystemTrayIcon::showMessage(title, text,
			                    (success ? QSystemTrayIcon::Information
			                             : QSystemTrayIcon::Warning));
}

void Applet::checkDaemonConnection()
{
	QDBusReply<bool> reply = daemon.call("reply");

	if ( m_daemonOk == reply.isValid() )
		return;
	
	if ( reply.isValid() )
	{
		checkState(false);
	}
	else
	{
		setIcon(iconNoDaemon);
		
		m_monitor->daemonError();
		
		QString title = tr("Daemon connection lost");
		QString text  = tr("The applet has lost the connection "
				           "with UbuDSL daemon. Probably the daemon "
				           "had been terminated.");
		QSystemTrayIcon::showMessage(title, text, QSystemTrayIcon::Warning);

		setToolTip(tr("Daemon connection lost"));
	}
	
	m_daemonOk = reply.isValid();
}

void Applet::connecting()
{
	static bool a = true;
	
	if (a)
		setIcon(iconConnecting);
	else
		setIcon(iconConnected);
	
	a = !a;
}

void Applet::playErrorSound()
{
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
	mediaObject->setCurrentSource(sndError);
	mediaObject->play();
#endif
}

void Applet::invalidAccountData()
{
	m_connectionTimer->stop();
	m_actionConnect->setIcon(iconConnected);
	
	setIcon(iconDisconnected);

	QString title = "Invalid login and/or password!";
	QString text  = "You have entered an invalid login and/or password "
		            "and the connection could not be established. Please "
		            "open the Settings dialog (right click on the Applet's "
		            "icon and choose the right position from the menu, or, "
		            "click the Open Settings button below) and "
		            "change your service's login and pssword to correct ones. "
		            "After you did this, you can try to connect again.";
	
	QMessageBox msgBox(QMessageBox::Critical, title, text, QMessageBox::Ok);
	msgBox.setAttribute(Qt::WA_QuitOnClose, false);
	
	QAbstractButton *settingsButton = msgBox.addButton("&Open Settings", QMessageBox::ActionRole);
	
	msgBox.exec();
	
	if (msgBox.clickedButton() == settingsButton)
		emit showSettings();
}

void Applet::compilingDriver()
{
	showMessage(CompilingDriver);

	m_actionConnect->setEnabled(false);
	m_monitor->setConnectEnabled(false);
}

void Applet::compilationFinished(const bool success)
{
	m_actionConnect->setEnabled(true);
	m_monitor->setConnectEnabled(true);

	if (success)
		showMessage(CompilationSucceed);
	else
		showMessage(CompilationFailed);
}
