#ifndef APPLET_H
#define APPLET_H

#include <QtGui/QWidget>
#include <QSystemTrayIcon>
#include <QMenu>
#include <QIcon>
#include <QAction>
#include <QRect>
#include <QDesktopWidget>
#include <QTimer>
#include <QDBusConnection>
#include <QDBusReply>
#include <QPixmap>
#include <QIcon>
#include <QMessageBox>
#include <QAbstractButton>

#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
#include <phonon>
#endif

#include "Monitor.h"
#include "GenerateLog.h"
#include "Settings.h"
#include "CustomStyle.h"

class Applet : public QSystemTrayIcon
{
    Q_OBJECT

public:
    Applet(QWidget *parent = 0);
    ~Applet();
    
private:
	enum Message
	{
		Connecting,
		ConnectionTimeout,
		RestartingModem,
		ConnectionTerminated,
		NoDaemon,
		IncorrectDNS,
		OpenDNS_Success,
		OpenDNS_Failed,
		Synchronizing,
		SynchronizationFailed,
		CompilingDriver,
		CompilationFailed,
		CompilationSucceed
	};
	
	void showMessage( Message msg );
	
	QDBusInterface daemon;
    
	bool m_daemonOk;
    
	QMenu   *m_menu;
	QAction *m_actionQuit;
	QAction *m_actionHide;
	QAction *m_actionGenerateLog;
	QAction *m_actionSettings;
	QAction *m_actionConnect;
	QAction *m_actionReconnect;
	QAction *m_actionRestartModem;
	QAction *m_separator1;
	QAction *m_separator2;
	
	QIcon iconConnected;
	QIcon iconConnecting;
	QIcon iconDisconnected;
	QIcon iconHide;
	QIcon iconNoDaemon;
	QIcon iconNoModem;
	QIcon iconShow;
	
	QMessageBox     *m_newModemMsgBox;
	QAbstractButton *m_btnYes;
	
	QTimer *m_connectionTimer;
	QTimer *m_timer;
	
#if QT_VERSION >= 0x040400 && PHONON_VERSION >= 0x040000
	Phonon::AudioOutput *audioOutput;
	Phonon::MediaObject *mediaObject;
	Phonon::MediaSource sndConnected;
	Phonon::MediaSource sndDisconnected;
	Phonon::MediaSource sndError;
	Phonon::MediaSource sndPlugged;
	Phonon::MediaSource sndUnplugged;
#endif
	
	Monitor     *m_monitor;
	GenerateLog *m_generateLog;
	Settings    *m_settingsDialog;

private slots:
	void showHideMonitor();
	void clicked( QSystemTrayIcon::ActivationReason reason );
	void checkState(bool firstTime = true);
	void enableGenerateLog();
	void disableGenerateLog();
	void establishingConnection();
	void connectionTimeout() { showMessage( ConnectionTimeout ); connectionTerminated(true); }
	void connectionEstablished();
	void restartModemStarted() { showMessage(RestartingModem); m_actionRestartModem->setEnabled(false); }
	void restartModemFinished() { m_actionRestartModem->setEnabled(true); }
	void restartModem();
	void connectionTerminated( bool justRun = false );
	void showSettings();
	void applySettings();
	void checkDaemonConnection();
	void connecting();
	void playErrorSound();
	void synchronizing() { showMessage( Synchronizing ); }
	void synchronizationFailed() { showMessage( SynchronizationFailed ); }
	void modemPlugged( const QString &name );
	void newModemPlugged( const QString &name );
	void modemUnplugged();
	void incorrectDNS();
	void settingOpenDNS( const bool success );
	void newModemUnplugged();
	void accountDataChanged(const bool success);
	void invalidAccountData();
	void compilingDriver();
	void compilationFinished(const bool finished);
	
signals:
	void quit();
};

#endif // APPLET_H
