#include "Monitor.h"
#include <QDebug>

Monitor::Monitor(QWidget *parent)
    : QMainWindow(parent),
    daemon( "com.ubudsl.Daemon", "/com/ubudsl/Daemon" , "com.ubudsl.Daemon", QDBusConnection::systemBus() ),
    m_strConnected(tr("connected")),
    m_strConnecting(tr("connecting")),
    m_strDisconnected(tr("disconnected")) 
{
	ui.setupUi(this);
	
	setWindowIcon(QIcon(":/images/connected.png"));
	
	setConnectButtonText(tr("&Connect"));
	
	setStyleSheet("QMainWindow{	background: url(\":/images/background.png\");}"
				  "QGroupBox::flat{	border-top: 1.5px solid qlineargradient(x1: 0, y1: 0, x2: 1, y2: 0,"
	                                                                  		"stop: 0 rgba(66, 66, 66, 0),"
	                                                                  		"stop: 0.3 rgb(66, 66, 66),"
	                                                                  		"stop: 0.7 rgb(66, 66, 66),"
	                                                                  		"stop: 1.0 rgba(66, 66, 66, 0));"
				  "margin-top: 1ex;}"

				  "QGroupBox::title	{ subcontrol-origin: margin; subcontrol-position: top center; padding: 0 3px;}"

				  "QPushButton { background: black;	border: 20px solid;	color: white; font: bold; left: 10px;}"

				  "QPushButton:pressed { background: qradialgradient(cx:0.5, cy:0.5, radius: 1, fx:0.5, fy:0.5, stop:0 rgba(255,255,255,200), stop:0.5 black);"
										 "border: 0; color: white; font: bold; min-width: 80px;}");
	
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "transmissionSpeed",
	                                     this, SLOT(transmissionSpeed(const unsigned int &, const unsigned int &)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "transmissionErrors",
	                                     this, SLOT(transmissionErrors(const unsigned int &, const unsigned int &)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "transmissionBytes",
	                                     this, SLOT(transmissionBytes(const double &, const double &)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "defaultRoute",
	                                     this, SLOT(setDefaultRoute(const QString &)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "IP",
	                                     this, SLOT(setIP(const QString &)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "connectionTerminated",
	                                     this, SLOT(connectionTerminated()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "connectionEstablished",
	                                     this, SLOT(connectionEstablished()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "establishingConnection",
	                                     this, SLOT(establishingConnection()));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "connectionState",
	                                     this, SLOT(setConnectionState(const int &)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "connectionTime",
	                                     this, SLOT(setConnectionTime( const int &, const int &, const int & )));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "setEnabledConnect",
			                             this, SIGNAL(enableConnectButton(bool)));
	QDBusConnection::systemBus().connect("", "", "com.ubudsl.Daemon", "setEnabledConnect",
			                             ui.btnConnect, SIGNAL(setEnabled(bool)));
	
	
	connect(ui.btnConnect, SIGNAL(clicked()), this, SLOT(connectDisconnect()));
	connect(ui.btnHide, SIGNAL(clicked()), this, SIGNAL(showHide()));

	getConnectionState();
	
	if ( m_state == ConnectionState::Connected )
	{
		getIP();
		getDefaultRoute();
	}
	
	if (daemon.isValid())
		qDebug() << "valid";
	else
		qDebug() << "invalid";
}

Monitor::~Monitor()
{

}

void Monitor::transmissionSpeed( const unsigned int &rx, const unsigned int &tx )
{
	ui.download->setText( rx < 1024 ? QString("%1 B/s").arg(rx) : QString("%1 KB/s").arg(rx/1024) );
	ui.upload->setText( tx < 1024 ? QString("%1 B/s").arg(tx) : QString("%1 KB/s").arg(tx/1024) );
}

void Monitor::transmissionErrors( const unsigned int &rx, const unsigned int &tx )
{
	ui.rx_errors->setText(QString("%1").arg(rx));
	ui.tx_errors->setText(QString("%1").arg(tx));
}

void Monitor::transmissionBytes( const double &rx, const double &tx )
{
	ui.rx->setText(QString("%1 MB").arg(rx/1048576, 0, 'f', 2));
	ui.tx->setText(QString("%1 MB").arg(tx/1048576, 0, 'f', 2));
}

void Monitor::getIP()
{
	QDBusReply<QString> ip = daemon.call("getIP");
	
	if (ip.isValid())
		setIP(ip);
	else
		setIP("...");
}

void Monitor::getDefaultRoute()
{
	QDBusReply<QString> route = daemon.call("getDefaultRoute");
	
	if (route.isValid())
		setDefaultRoute(route);
	else
		setDefaultRoute("...");
}

void Monitor::getConnectionState()
{
	QDBusReply<int> state = daemon.call("getConnectionState");
	
	if (state.isValid())
		setConnectionState(state);
	else
		setConnectionState(ConnectionState::Disconnected);
}

void Monitor::setIP( const QString &ip )
{
	ui.ip->setText( ip );
}

void Monitor::setDefaultRoute( const QString &iface )
{
	ui.route->setText( iface );
}

void Monitor::setConnectionState( const int &state )
{
	if (state == ConnectionState::Connected)
	{
		qDebug() << "connected";
		ui.state->setText( m_strConnected );
	}
	else if (state == ConnectionState::Connecting)
	{
		qDebug() << "connecting";
		ui.state->setText( m_strConnecting );
	}
	else if (state == ConnectionState::Disconnected)
	{
		qDebug() << "disconnected";
		ui.state->setText( m_strDisconnected );
	}
	
	m_state = state;
}

void Monitor::setConnectionTime( const int &h, const int &m, const int &s )
{
	QTime time( h, m, s );
	
	if ( time.isNull() )
		ui.time->setText("...");
	else
		ui.time->setText( time.toString("HH:mm:ss") );
}

void Monitor::closeEvent( QCloseEvent *e )
{
	emit showHide();
	e->ignore();
}

void Monitor::connectDisconnect()
{
	if ( m_state == ConnectionState::Connected )
	{
		disconnect_();
	}
	else if ( m_state == ConnectionState::Disconnected )
	{
		connect_();
	}
	else
	{
		getConnectionState();
	}

}

void Monitor::connect_()
{
	daemon.call("establishConnection");
}

void Monitor::disconnect_()
{
	daemon.call("terminateConnection");
}

void Monitor::reconnect()
{
	disconnect_();
	
	QTimer::singleShot( 1000, this, SLOT(connect_()) );
}

void Monitor::resetLabels()
{
	ui.ip->setText("...");
	ui.route->setText("...");
	ui.rx->setText("...");
	ui.rx_errors->setText("...");
	ui.tx->setText("...");
	ui.tx_errors->setText("...");
	ui.state->setText("...");
	ui.time->setText("...");
	ui.download->setText("...");
	ui.upload->setText("...");
}

void Monitor::connectionTerminated()
{
	resetLabels();
	setConnectionState(ConnectionState::Disconnected);
}

void Monitor::establishingConnection()
{
	resetLabels();
	setConnectionState(ConnectionState::Connecting);
}

void Monitor::connectionEstablished()
{
	setConnectionState(ConnectionState::Connected);
}

void Monitor::daemonError()
{
	ui.state->setText( tr("error") );
	ui.time->setText( "..." );
	ui.ip->setText( "..." );
	ui.route->setText( "..." );
	ui.download->setText( "..." );
	ui.upload->setText( "..." );
	ui.tx->setText( "..." );
	ui.rx->setText( "..." );
	ui.tx_errors->setText( "..." );
	ui.rx_errors->setText( "..." );
}

void Monitor::on_btnAbout_clicked()
{
	QDialog about;
	Ui::AboutClass ui;
	ui.setupUi(&about);

	QFile licence(":LICENSE");
	if (licence.open(QFile::ReadOnly))
	{
		ui.labelLicence->setText(licence.readAll());
	}

	about.exec();
}
