#ifndef MONITOR_H
#define MONITOR_H

#include <QtGui/QMainWindow>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QString>
#include <QDBusReply>
#include <QTime>
#include <QTimer>
#include <QCloseEvent>
#include <QMessageBox>
#include <QFile>
#include <QLocale>

#include "libubudsl/Globals.h"

#include "ui_Monitor.h"
#include "ui_About.h"

class Monitor : public QMainWindow
{
    Q_OBJECT

public:
    Monitor(QWidget *parent = 0);
    ~Monitor();
    
    void daemonError();
    
public slots:
	void connectDisconnect();
	void setConnectButtonText(const QString &text) { ui.btnConnect->setText(text); }
	void setConnectEnabled( bool enabled) { ui.btnConnect->setEnabled(enabled); }
	void reconnect();
	
private slots:
	void transmissionSpeed( const unsigned int &rx, const unsigned int &tx );
	void transmissionErrors( const unsigned int &rx, const unsigned int &tx );
	void transmissionBytes( const double &rx, const double &tx );
	void setIP( const QString &ip );
	void setDefaultRoute( const QString &iface );
	void setConnectionState( const int &state );
	void setConnectionTime( const int &h, const int &m, const int &s );
	virtual void closeEvent( QCloseEvent *e );
	void connectionTerminated();
	void establishingConnection();
	void connectionEstablished();
	void on_btnAbout_clicked();
	void connect_();
	void disconnect_();

private:
	void getIP();
	void getDefaultRoute();
	void getConnectionState();
	void resetLabels();

    Ui::MonitorClass ui;
    QDBusInterface daemon;
    int m_state;
    QString m_strConnected;
    QString m_strConnecting;
    QString m_strDisconnected;
    
signals:
	void showHide();
	void enableConnectButton(bool enable);
};

#endif // MONITOR_H
