#include "Settings.h"

Settings::Settings( QWidget *parent)
    : QDialog(parent)
{
	ui.setupUi(this);
	setAttribute(Qt::WA_QuitOnClose, false);

	connect(ui.leLogin, SIGNAL(textChanged(const QString &)), this, SLOT(dataChanged()));
	connect(ui.lePassword, SIGNAL(textChanged(const QString &)), this, SLOT(dataChanged()));
	connect(ui.cbConnectAtBoot, SIGNAL(toggled(bool)), this, SLOT(dataChanged()));
	connect(ui.cbConnectAtModemPlug, SIGNAL(toggled(bool)), this, SLOT(dataChanged()));
	connect(ui.cbIdle, SIGNAL(toggled(bool)), this, SLOT(dataChanged()));
	connect(ui.spinIdle, SIGNAL(valueChanged(int)), this, SLOT(dataChanged()));
}

Settings::~Settings()
{

}

void Settings::getData()
{
	m_idle = ui.spinIdle->value();
	m_login = ui.leLogin->text();
	m_password = ui.lePassword->text();
	m_idleChecked = ui.cbIdle->isChecked();
	m_connectAtBoot = ui.cbConnectAtBoot->isChecked();
	m_connectAtModemPlug = ui.cbConnectAtModemPlug->isChecked();
	
	ui.buttonSave->setEnabled(false);
}

void Settings::dataChanged()
{
	bool idle = (m_idleChecked == ui.cbIdle->isChecked() && m_idle == ui.spinIdle->value() || !ui.cbIdle->isChecked());

	if ( idle &&
		 m_login == ui.leLogin->text() &&
		 m_password == ui.lePassword->text() &&
		 m_connectAtBoot == ui.cbConnectAtBoot->isChecked() &&
		 m_connectAtModemPlug == ui.cbConnectAtModemPlug->isChecked() )
	{
		ui.buttonSave->setEnabled(false);
	}
	else
	{
		ui.buttonSave->setEnabled(true);
	}
}

bool Settings::getAccountDataChanged()
{
	return ( m_login != ui.leLogin->text() || m_password != ui.lePassword->text() );
}

bool Settings::getConnectAtBootChanged()
{
	return m_connectAtBoot != ui.cbConnectAtBoot->isChecked();
}

bool Settings::getConnectAtModemPlugChanged()
{
	return m_connectAtModemPlug != ui.cbConnectAtModemPlug->isChecked();
}

bool Settings::getIdleTimeChanged()
{
	return (m_idle != ui.spinIdle->value() || m_idleChecked != ui.cbIdle->isChecked());
}

bool Settings::getIdleChanged()
{
	return m_idleChecked != ui.cbIdle->isChecked();
}
