#ifndef LOGGENERATOR_H
#define LOGGENERATOR_H

#include <QtGui/QWidget>
#include <QWizard>
#include <QWizardPage>
#include <QLabel>
#include <QRadioButton>
#include <QVBoxLayout>
#include <QTextEdit>
#include <QString>
#include <QDebug>
#include <QAbstractButton>
#include <QList>
#include <QMap>
#include <QMapIterator>
#include <QTextBrowser>

#include "GeneratorThread.h"

class IntroPage: public QWizardPage
{
    Q_OBJECT
public:
    IntroPage(QWidget *parent = 0);

private:
    QLabel *labelTop;
};

class QuestionYesNoPage: public QWizardPage
{
    Q_OBJECT
public:
	QuestionYesNoPage(const QString &question, const int &currQuestion, const int &questionsCount, QWidget *parent = 0);

	bool getYesOrNo() { return radioYes->isChecked(); }
private:
    QLabel *labelTop;
    QRadioButton *radioYes;
    QRadioButton *radioNo;
};

class SummaryPage: public QWizardPage
{
    Q_OBJECT
public:
	SummaryPage(QWidget *parent = 0);

	QString getSummary() { return textEdit->toPlainText(); }
private:
    QLabel *labelTop;
    QLabel *labelLng;
    QTextEdit *textEdit;
signals:
	void textChanged();
};

class ReviewPage: public QWizardPage
{
    Q_OBJECT
public:
	ReviewPage(QWidget *parent = 0);
	
	void updateData( QMap<QString, bool> &map );
private:
    QLabel *labelTop;
    QTextBrowser *textBrowser;
};

class FinishPage: public QWizardPage
{
    Q_OBJECT
public:
	FinishPage(QAbstractButton *buttonFinish, GeneratorThread *gThread, QWidget *parent = 0);

private:
    QLabel *labelTop;
    QLabel *labelFinish;
    QLabel *labelFailed;
    QAbstractButton *buttonFinish;
    GeneratorThread *gThread;
private slots:
	void generatingFinished();
	void generatingFailed();
};


class LogGenerator : public QWizard
{
    Q_OBJECT
public:
	enum Pages
	{ Intro_Page,
	  Q1_Page,
	  Q2_Page,
	  Q3_Page,
      Q4_Page,
      Q5_Page,
      Summary_Page,
      Review_Page,
      Finish_Page
	}
	;
    LogGenerator(QWidget *parent = 0);
    ~LogGenerator();

private:
	QString q1;
	QString q2;
	QString q3;
	QString q4;
	QString q5;
	
    QuestionYesNoPage *pageQ1;
    QuestionYesNoPage *pageQ2;
    QuestionYesNoPage *pageQ3;
    QuestionYesNoPage *pageQ4;
    QuestionYesNoPage *pageQ5;
	SummaryPage       *summaryPage;
	ReviewPage        *reviewPage;
	FinishPage        *finishPage;
	
	GeneratorThread *gThread;
	
private slots:
	void currentIdChanged(int id);
	void summaryChanged();
	QString getGatheredInfo();
};

#endif // LOGGENERATOR_H
