/*
 * File: FileLocker.cpp
 * Created: Mar 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "FileLocker.h"

int lockFile ( const QString &file )
{
    errno = 0;

    int FD = open ( qPrintable ( file ), O_RDWR | O_CREAT | O_TRUNC, 0640 );
    if ( FD < 0 )
    {
        // Read only .. cant have locking problems there.
        if ( errno == EROFS )
        {
        	if ( debug )
        		fprintf( pFile, "getFileLock(): Not using locking for read only lock file %s\n", qPrintable(file) );
        	
            return dup ( 0 );    // Need something for the caller to close
        }

        if ( debug )
        	fprintf( pFile, "getFileLock(): Could not open lock file %s\n", qPrintable(file) );

        // Feh.. We do this to distinguish the lock vs open case..
        errno = EPERM;
        return -1;
    }

    setCloseExec ( FD, true );

    // Aquire a write lock
    struct flock fl;
    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    fl.l_start = 0;
    fl.l_len = 0;
    if ( fcntl ( FD, F_SETLK, &fl ) == -1 )
    {
        if ( errno == ENOLCK )
        {
            if ( debug )
            	fprintf( pFile, "getFileLock(): Not using locking for nfs mounted lock file %s\n", qPrintable(file) );

            return dup ( 0 );    // Need something for the caller to close
        }
        
        if ( debug )
             fprintf( pFile, "getFileLock(): Could not get lock: %s\n", qPrintable(file) );

        close ( FD );
        return -1;
    }
    return FD;
}


void setCloseExec ( int Fd, bool Close )
{
    if ( fcntl ( Fd, F_SETFD, ( Close == false ) ? 0 : FD_CLOEXEC ) != 0 )
    {
    	if ( debug )
    		fprintf( pFile, "getFileLock(): Could not set close on exec %s\n", strerror( errno ) );
//        exit(100);
    }
}

void unlockFile( int lockfd )
{
	struct flock lock;
	
	lock.l_type = F_UNLCK;
	lock.l_whence = 0;
	lock.l_start = 0;
	lock.l_len = 0;

	if ( debug )
		fprintf( pFile, "unlockFile(): unlocking fd: %i\n", lockfd );


	if (fcntl(lockfd, F_SETLK, &lock) == -1 && debug )
		fprintf( pFile, "unlockFile(): fcntl unlock failed on fd %i: %s\n", lockfd, strerror(errno) );

	if (close(lockfd) && debug )
		fprintf( pFile, "unlockFile(): lock file close failed on fd %i: %s\n", lockfd, strerror(errno) );
}
