/*
 * File: ModemManager.cpp
 * Created: Fri 08 Feb 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "ModemManager.h"

ModemManager::ModemManager()
        : vendorID ( 0 ),
        productID ( 0 )
{
    deviceTimer = new QTimer;
    connect ( deviceTimer, SIGNAL ( timeout() ), this, SLOT ( verifyDevice() ) );

    initModemsMap();


}

void ModemManager::initModemsMap()
{
    Modem sagem_fast_800_e4;
    sagem_fast_800_e4.name = "Sagem F@st 800 E4";
    sagem_fast_800_e4.driver_name = "ueagle-atm";
    sagem_fast_800_e4.driver_version = "1.4";
    sagem_fast_800_e4.usb_id.insert ( 0x1110, 0x9041 );
    sagem_fast_800_e4.usb_id.insert ( 0x1110, 0x9042 );

    Modem sagem_fast_800;
    sagem_fast_800.name = "Sagem F@st 800";
    sagem_fast_800.driver_name = "ueagle-atm";
    sagem_fast_800_e4.driver_version = "default";
    sagem_fast_800.usb_id.insert ( 0x1110, 0x9010 );
    sagem_fast_800.usb_id.insert ( 0x1110, 0x900f );
    sagem_fast_800.usb_id.insert ( 0x1110, 0x9021 );
    sagem_fast_800.usb_id.insert ( 0x1110, 0x9022 );
    sagem_fast_800.usb_id.insert ( 0x1110, 0x9023 );
    sagem_fast_800.usb_id.insert ( 0x1110, 0x9024 );
    sagem_fast_800.usb_id.insert ( 0x1110, 0x9031 );
    sagem_fast_800.usb_id.insert ( 0x1110, 0x9032 );

    Modem thomson_speedtouch_330;
    thomson_speedtouch_330.name = "Thomson Speedtouch 330";
    thomson_speedtouch_330.driver_name = "speedtch";
    thomson_speedtouch_330.driver_version = "default";
    thomson_speedtouch_330.usb_id.insert ( 0x06b9, 0x4061 );

    Modem zte_zxdsl_852;
    zte_zxdsl_852.name = "ZTE ZXDSL 852";
    zte_zxdsl_852.driver_name = "unicorn";
    zte_zxdsl_852.driver_version = "neo";
    zte_zxdsl_852.usb_id.insert ( 0x0483, 0x0138 );

    Modem cxacru_based;
    cxacru_based.name = "CXACRU modem";
    cxacru_based.driver_name = "cxacru";
    cxacru_based.driver_version = "neo";
    cxacru_based.usb_id.insert ( 0x0572, 0xcafe );
    cxacru_based.usb_id.insert ( 0x0572, 0xcb00 );
    cxacru_based.usb_id.insert ( 0x0572, 0xcb01 );
    cxacru_based.usb_id.insert ( 0x0572, 0xcb02 );
    cxacru_based.usb_id.insert ( 0x0572, 0xcb06 );
    cxacru_based.usb_id.insert ( 0x0572, 0xcb07 );
    cxacru_based.usb_id.insert ( 0x08e3, 0x0100 );
    cxacru_based.usb_id.insert ( 0x08e3, 0x0102 );
    cxacru_based.usb_id.insert ( 0x0eb0, 0x3457 );
    cxacru_based.usb_id.insert ( 0x1803, 0x5510 );
    cxacru_based.usb_id.insert ( 0x0675, 0x0200 );
    cxacru_based.usb_id.insert ( 0x0586, 0x330a );
    cxacru_based.usb_id.insert ( 0x0586, 0x330b );
    cxacru_based.usb_id.insert ( 0x0659, 0x0020 );
    cxacru_based.usb_id.insert ( 0x0509, 0x0812 );
    cxacru_based.usb_id.insert ( 0x100d, 0xcb01 );
    cxacru_based.usb_id.insert ( 0x100d, 0x3342 );

    m_modems.append ( sagem_fast_800_e4 );
    m_modems.append ( sagem_fast_800 );
    m_modems.append ( thomson_speedtouch_330 );
    m_modems.append ( zte_zxdsl_852 );
    m_modems.append ( cxacru_based );
}

void ModemManager::connectToDBus()
{
    qDebug() << "Connecting to DBus...";
    QDBusConnection::systemBus().connect ( QString(), QString(),
                                           "org.freedesktop.Hal.Manager", "DeviceAdded",
                                           this, SLOT ( deviceAdded ( QString ) ) );
}


void ModemManager::deviceAdded ( const QString &udi )
{
    qDebug() << "New device:" << udi;

    QDBusInterface iface ( "org.freedesktop.Hal", udi,
                           "org.freedesktop.Hal.Device",
                           QDBusConnection::systemBus() );

    QDBusReply<int> p_id = iface.call ( "GetProperty", "usb_device.product_id" );
    QDBusReply<int> v_id = iface.call ( "GetProperty", "usb_device.vendor_id" );

    if ( p_id.isValid() && v_id.isValid() )
    {
        productID = p_id.value();
        vendorID  = v_id.value();
        lastAdded = udi;

        qDebug() << "p_id:" << p_id.value() << "v_id:" << v_id.value();

        deviceTimer->stop();
        deviceTimer->setSingleShot ( true );
        deviceTimer->start ( 5000 );
    }
}


void ModemManager::verifyDevice()
{
    qDebug() << "ModemManager::verifyDevice()";
    modemFound = false;

    QDBusInterface hal_iface ( "org.freedesktop.Hal", "/org/freedesktop/Hal/Manager",
                               "org.freedesktop.Hal.Manager", QDBusConnection::systemBus() );

    QDBusReply<bool> reply = hal_iface.call ( "DeviceExists", lastAdded );
    if ( !reply.isValid() || !reply.value() )
        return;

    QListIterator<Modem> modemIterator ( m_modems );
    while ( modemIterator.hasNext() )
    {
        Modem modem = modemIterator.next();
        qDebug() << "iterating through:" << modem.name;
        if ( modem.usb_id.contains ( vendorID, productID ) )
        {
            qDebug() << "contains";
            modemFound = true;
            modemName = modem.name;
            driverName = modem.driver_name;
            driverVersion = modem.driver_version;

            emit foundModem();
            break;
        }
    }
}


bool ModemManager::getIsModemFound()
{
    return modemFound;
}

QString ModemManager::getModemName()
{
    return modemName;
}

QString ModemManager::getDriverName()
{
    return driverName;
}

QString ModemManager::getDriverVersion()
{
    return driverVersion;
}

int ModemManager::getVendorID()
{
    return vendorID;
}


int ModemManager::getProductID()
{
    return productID;
}

QString ModemManager::getUDI()
{
	return lastAdded;
}
