/*
 * File: MyProcess.cpp
 * Last modified: Tue 29 Jan 2008
 * Author: Ricardo Villalba
 * Note: This file was taken from the SMplayer project
 */

#include "MyProcess.h"


MyProcess::MyProcess ( QObject * parent ) : QProcess ( parent )
{
    clearArguments();
    setProcessChannelMode ( QProcess::MergedChannels );

    connect ( this, SIGNAL ( readyReadStandardOutput() ), this, SLOT ( readStdOut() ) );

    connect ( this, SIGNAL ( finished ( int, QProcess::ExitStatus ) ),
              this, SLOT ( procFinished() ) );
}

void MyProcess::clearArguments()
{
    program = "";
    arg.clear();
}

bool MyProcess::isRunning()
{
    return ( state() == QProcess::Running );
}

void MyProcess::addArgument ( const QString & a )
{
    if ( program.isEmpty() )
    {
        program = a;
    }
    else
    {
        arg.append ( a );
    }
}

QStringList MyProcess::arguments()
{
    QStringList l = arg;
    l.prepend ( program );
    return l;
}

void MyProcess::start()
{
    remaining_output.clear();

    QProcess::start ( program, arg );
}

void MyProcess::start ( const QString &program, const QStringList &arg )
{
    remaining_output.clear();

    QProcess::start ( program, arg );
}

void MyProcess::readStdOut()
{
    genericRead ( readAllStandardOutput() );
}


void MyProcess::genericRead ( QByteArray buffer )
{
    QByteArray ba = remaining_output + buffer;
    int start = 0;
    int from_pos = 0;
    int pos = canReadLine ( ba, from_pos );

    //qDebug("MyProcess::read: pos: %d", pos);
    while ( pos > -1 )
    {
        // Readline
        //QByteArray line = ba.left(pos);
        QByteArray line = ba.mid ( start, pos - start );
        //ba = ba.mid(pos+1);
        from_pos = pos + 1;
        start = from_pos;

        emit lineAvailable ( line );

        pos = canReadLine ( ba, from_pos );
    }

    remaining_output = ba.mid ( from_pos );
}

int MyProcess::canReadLine ( const QByteArray & ba, int from )
{
    int pos1 = ba.indexOf ( '\n', from );
    int pos2 = ba.indexOf ( '\r', from );

    //qDebug("MyProcess::canReadLine: pos2: %d", pos2);

    if ( ( pos1 == -1 ) && ( pos2 == -1 ) ) return -1;

    int pos = pos1;
    if ( ( pos1 != -1 ) && ( pos2 != -1 ) )
    {
        /*
        if (pos2 == (pos1+1)) pos = pos2; // \r\n
        else
        */
        if ( pos1 < pos2 ) pos = pos1;
        else pos = pos2;
    }
    else
    {
        if ( pos1 == -1 ) pos = pos2;
        else
            if ( pos2 == -1 ) pos = pos1;
    }

    return pos;
}

/*!
Do some clean up, and be sure that all output has been read.
*/
void MyProcess::procFinished()
{
    //qDebug ( "MyProcess::procFinished" );

    //qDebug("MyProcess::procFinished: Bytes available: %ld", bytesAvailable());
    if ( bytesAvailable() > 0 ) readStdOut();
}

//#include "moc_MyProcess.cpp"
