/*
 * File: PackageManager.cpp
 * Created: Wed 30 Jan 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "PackageManager.h"

extern bool  debug;
extern FILE *pFile;

PackageManager::PackageManager ( const QString &manager, QThread *parent )
        : QObject(parent),
          m_manager ( manager )
{
    qDebug() << "PackageManager::parent:" << parent << "\nP.'s thread:" << parent->thread();
    apt = new Apt(parent);
    zypper = new Zypper(parent);
    printf("z:1\n");
    if ( parent != NULL )
    {
    	//apt->moveToThread(parent->thread());
    	//zypper->moveToThread(parent->thread());
    }
    printf("z:2\n");

    qDebug() << "m_manager" << m_manager;
}


PackageManager::~PackageManager()
{
// qDebug() << "~PackageManager()";
//
// //check whether directory for logs exist
// //and if it doesn't - create a new one
//    QDir *tmpDir = new QDir( "/opt/ubudsl/logs" );
//    if ( !tmpDir->exists() )
//    {
//        qDebug() << "Logs directory does not exist, creating a new one.";
//        tmpDir->mkpath( tmpDir->path() );
//    }
//
//    delete tmpDir;
//
//    //create (replace if exists) and write the class's log file
//    QFile *packageManagerLog = new QFile( "/opt/ubudsl/logs/packagemanager.log" );
//    if ( packageManagerLog->open(QFile::WriteOnly|QFile::Truncate) )
//    {
//        packageManagerLog->write( packageManagerDebug );
//        packageManagerLog->close();
//    }
//    else
//     qDebug() << "~PackageManager: Could not open log file...";
//
//    delete packageManagerLog;
//    delete parser;
}

int PackageManager::addCD ( const QString &mntPoint )
{
    if ( m_manager == "apt" )
    {
        return apt->addCD ( mntPoint );
    }
    else if ( m_manager == "zypper" )
    {
        QString productFilePath;

        FileManager fm;
        fm.findFile ( mntPoint, "products", productFilePath );

        if ( productFilePath.isEmpty() )
        {
            qDebug() << "product file not found, aborting";
            return -1;
        }

        QFile product ( productFilePath );
        if ( product.open ( QFile::ReadOnly ) )
        {
            qDebug() << "product file is open";
            QRegExp productRegExp ( "^/\\s+(.{1,100})\\s*$" );
            QString line = product.readLine ( 100 );

            if ( productRegExp.exactMatch ( line ) )
            {
                qDebug() << "exact match:" << productRegExp.cap ( 1 ).trimmed();
                QList<ZypperRepo> repos = zypper->getRepos();
                QListIterator<ZypperRepo> it ( repos );

                while ( it.hasNext() )
                {
                    ZypperRepo repo = it.next();
                    qDebug() << "iterating, current repo:" << repo.name;
                    if ( repo.name == productRegExp.cap ( 1 ).trimmed() )
                    {
                        if ( repo.enabled )
                        {
                            qDebug() << "repo is already enabled";
                            return 0;
                        }
                        else
                        {
                            qDebug() << "enabling repo:" << repo.alias;
                            return zypper->enableRepo ( repo.alias );
                        }
                    }
                }

                return zypper->addRepo ( mntPoint, productRegExp.cap ( 1 ).trimmed() );
            }
        }
        else
        {
            qDebug() << "product file not open, adding \"UbuDSL added CD/DVD\"";
            return zypper->addRepo ( mntPoint, "UbuDSL added CD/DVD" );
        }
    }
    return -2;
}
int PackageManager::installPackages ( const QStringList &pkgs )
{
    if ( m_manager.isEmpty() || pkgs.isEmpty() )
        return 0;

    if ( m_manager == "apt" )
    {
        qDebug() << "PackageManager: using Apt";
        return apt->install ( pkgs, false );
    }
    else if ( m_manager == "zypper" )
    {
        qDebug() << "PackageManager: using Zypper";
        return zypper->install ( pkgs );
    }
    else
    {
        qDebug() << "PackageManager: unknown m_manager, aborting! (" << m_manager << ")";
        return -2;
    }
    
    return -3;
}

bool PackageManager::getIsPackageInstalled ( const QString &pkg )
{
    if ( m_manager == "apt" )
        return apt->getIsPackageInstalled ( pkg );
    else if ( m_manager == "zypper" )
        return zypper->getIsPackageInstalled ( pkg );
    else
        return false; //not too complex solution
}

bool PackageManager::getArePackagesAvailable ( const QStringList pkgs )
{
	printf ("PM::gapa: %s\n", qPrintable( pkgs.join(", ")) );
	
    if ( pkgs.isEmpty() )
    {
    	printf("PM::getArePackagesAvailable: empty list!\n");
        return true;
    }

    if ( m_manager == "apt" )
    {
    	if ( debug )
    		fprintf( pFile, "return apt->getArePackagesAvailable ( pkgs );\n");

        return apt->getArePackagesAvailable ( pkgs );
    }
    else if ( m_manager == "zypper" )
    {
    	if ( debug )
    		fprintf( pFile, "return zypper->getArePackagesAvailable ( pkgs );\n");
        return zypper->getArePackagesAvailable ( pkgs );
    }
    else
    {
    	fprintf( pFile, "PackageManager::getArePackagesAvailable: unknown package manager!\n");
    }
    
    printf("return true;\n");
    return true;
}

QStringList PackageManager::getNotInstalled ( const QStringList pkgs )
{
    if ( m_manager == "apt" )
        return apt->getNotInstalled ( pkgs );
    else if ( m_manager == "zypper" )
        return zypper->getNotInstalled ( pkgs );
    else
    {
    	printf("PackageManager::getNotInstalled(): return pkgs;\n");
        return pkgs;
    }
}
