/*
 * File: Rpm.cpp
 * Created: Apr 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "Rpm.h"

Rpm::Rpm( QThread *parent )
:QObject(parent),
 m_askProcess(new QProcess(0))
{
    QStringList env = QProcess::systemEnvironment();
    env.append( "LC_ALL=C" );

    m_askProcess->setEnvironment( env );
    
	if ( parent != NULL )
	{
		m_askProcess->moveToThread(parent);
	}
}


Rpm::~Rpm()
{
	delete m_askProcess;
}


int Rpm::queryDB()
{
    m_askProcess->start ( "rpm", QStringList() << "-qa" );

    if ( m_askProcess->waitForFinished() )
    {
        readOutput = m_askProcess->readAll();
        installedPkgs = readOutput.split ( '\n' );
    }

    int exitCode = m_askProcess->exitCode();

    qDebug() << "Rpm::queryDB(): m_askProcess exited with code:" << exitCode;

    return exitCode;
}


QStringList Rpm::getNotInstalled ( QStringList pkgs )
{
    if ( pkgs.isEmpty() )
        return QStringList();
    else if ( installedPkgs.isEmpty() )
        return pkgs;

    QListIterator<QByteArray> it ( installedPkgs );

    while ( it.hasNext() )
    {
        QByteArray ba = it.next();
        foreach ( QString pkg, pkgs )
        {
            if ( ba.startsWith ( qPrintable ( pkg ) ) )
            {
                qDebug() << pkg << "is installed.";
                pkgs.removeAll ( pkg );
            }
        }
    }

    return pkgs;
}
