/*
 * File: ServiceParser.cpp
 * Created: Thu 07 Feb 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "ServiceParser.h"

ServiceParser::ServiceParser ( const QString &xmlFile )
{
    ispListFile.setFileName ( xmlFile );

    if ( ispListFile.exists() )
    {
        qDebug() << "Using external file:" << xmlFile;
    }

    domdocument.setContent ( &ispListFile, true );

    rootElement  = domdocument.documentElement ( );
    mainElements = rootElement.elementsByTagName ( "country" );

    n = mainElements.count();//gets number of "country" tags
}


void ServiceParser::getCountries()
{
    qDebug() << "Found \"country\" tags (n):" << n;//debug: checks 'n'
    QMap<int, QString> cMap;

    for ( int i = 0; i < n; i++ )
    {
        QString country = mainElements.item ( i ).toElement().attribute ( "code" );
        emit foundCountry ( country );
        cMap.insert ( i + 1, country );
    }

    emit countriesMap ( cMap );
}



void ServiceParser::getServicesForCountry ( const QString &ccode )
{
    QMap<int, QString> sMap;

    currentCountry = ccode;
    for ( int i = 0; i < n; i++ )
    {
        attributeCode = mainElements.item ( i ).toElement().attribute ( "code" );

        if ( attributeCode == ccode )
        {
            serviceElements = mainElements.item ( i ).toElement().elementsByTagName ( "service" );
            int o           = serviceElements.count();//gets number of "country" tags

            for ( int j = 0; j < o; j++ )
            {
                serviceElement = serviceElements.item ( j ).toElement();
                QString service = serviceElement.attribute ( "name" );
                //emit foundService( serviceElement.attribute("service") );
                emit foundService ( service );

                sMap.insert ( j + 1, service );
            }
        }
    }

    emit servicesMap ( sMap );
}



void ServiceParser::requestServiceData ( const QString &service )
{
    for ( int i = 0; i < n; i++ )
    {
        attributeCode = mainElements.item ( i ).toElement().attribute ( "code" );
        //qDebug() << "attributeCode:" << attributeCode;

        if ( attributeCode == currentCountry )
        {
            serviceElements = mainElements.item ( i ).toElement().elementsByTagName ( "service" );
            int o           = serviceElements.count();//gets number of "country" tags

            for ( int j = 0; j < o; j++ )
            {
                vpiElement     = serviceElements.item ( j ).toElement();
                serviceElement = serviceElements.item ( j ).toElement();
                //qDebug() << "serviceElement: name:" << serviceElement.attribute("name");

                if ( serviceElement.attribute ( "name" ) == service )
                {
                    vpiElement        = serviceElement.firstChildElement ( "vpi" );
                    vciElement        = serviceElement.firstChildElement ( "vci" );
                    protocolElement   = serviceElement.firstChildElement ( "protocol" );
                    dnsServer1Element = serviceElement.firstChildElement ( "dns_server_1" );
                    dnsServer2Element = serviceElement.firstChildElement ( "dns_server_2" );
                    VPI               = vpiElement.text().toUShort();
                    VCI               = vciElement.text().toUShort();
                    protocol          = protocolElement.text();

                    dnsServers.clear();

                    if (dnsServer1Element.text().isEmpty())
                        qDebug() << "DNS Server 1 is empty!";
                    else
                        dnsServers << dnsServer1Element.text();

                    if (dnsServer2Element.text().isEmpty())
                        qDebug() << "DNS Server 2 is empty!";
                    else
                        dnsServers << dnsServer2Element.text();

                    qDebug() << "VPI:" << VPI << "VCI:" << VCI << "PROTOCOL:" << protocol << "\nDNS Servers:" << dnsServers;
                    emit sendServiceData ( VPI, VCI, protocol, dnsServers );
                    break;
                }
            }
        }
    }
}
