/*
 * File: ListBar.cpp
 * Created: Mon 31 Mar 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "ListBar.h"

ListBar::ListBar ( Type t, QWidget *parent ) : QWidget ( parent )
{
    QFont font ( "Free Sans", 10, QFont::Bold );
    font.insertSubstitutions("Free Sans", QStringList() << "Nimbus Sans L" << "Luxi Sans" << "Lucida Sans" << "Andale Sans");

    this->t = t;

    c1  = QColor ( Qt::gray );
    c2  = QColor ( Qt::white );
    lbl = new QLabel ( this );
    op  = 0.0;

    lbl->setGeometry ( 15, 2, 165, 34 );
    lbl->setStyleSheet( "QLabel { color: rgb(81, 81, 81); }" );
    lbl->setFont ( font );
    lbl->setText ( "" );

    setMinimumSize ( QSize ( 200, 34 ) );
    setMaximumSize ( QSize ( 2000, 340 ) );

    setVisible ( false );
}


void ListBar::paintEvent ( QPaintEvent * )
{
    const QColor frColor = QColor ( 211, 206, 195 );
    const QColor bgColor = QColor ( 250, 247, 246 );

    QRadialGradient barBottomRg ( QPointF ( 90, 12 ), 80 );;
    barBottomRg.setColorAt ( 0, QColor ( 194, 189, 179 ) );
    barBottomRg.setColorAt ( 1, QColor ( 173, 170, 160, 80 ) );

    QRadialGradient ballTopRg ( QPointF ( 175, 5 ), 6 );
    ballTopRg.setColorAt ( 0, c1 );
    ballTopRg.setColorAt ( 1, c2 );
    ballTopRg.setSpread ( QGradient::ReflectSpread );

    QRadialGradient ballBottomRg ( QPointF ( 175, 17 ), 10 );
    ballBottomRg.setColorAt ( 0, QColor ( 194, 189, 179 ) );
    ballBottomRg.setColorAt ( 1, QColor ( 173, 170, 160, 80 ) );
    ballBottomRg.setSpread ( QGradient::ReflectSpread );

    QLinearGradient barTopLg ( 0.0, 0.00, 0.00, 34.0 );
    barTopLg.setColorAt ( 1.0, QColor ( 242, 236, 237 ) );
    barTopLg.setColorAt ( 0.0, QColor ( 254, 253, 251 ) );

    QBrush brush ( frColor );
    QBrush barBottomRgBrush ( barBottomRg );
    QBrush barTopLgBrush ( barTopLg );
    QBrush ballTopRgBrush ( ballTopRg );
    QBrush ballBottomRgBrush ( ballBottomRg );

    QRectF rectangle1;
    QRectF roundRect1;
    QRectF rectangle2;
    QRectF roundRect2;

    if ( t == Squared )
    {
        rectangle1.setRect ( 0.0, 0.0, 200.0, 34.5 );
        rectangle2.setRect ( 2.0, 2.0, 196.0, 29.5 );
    }
    else if ( t == RoundedTop )
    {
        rectangle1.setRect ( 0.0, 20.0, 200.0, 14.0 );
        rectangle2.setRect ( 2.0, 21.0, 196.0, 11.0 );
        roundRect1.setRect ( 0.0, 0.0 , 200.0, 34.0 );
        roundRect2.setRect ( 2.0, 1.5 , 196.0, 30.0 );
    }
    else if ( t == RoundedBottom )
    {
        rectangle1.setRect ( 0.0, 0.0 , 200.0, 20.0 );
        rectangle2.setRect ( 2.0, 1.5 , 196.0, 21.0 );
        roundRect1.setRect ( 0.0, 0.0 , 200.0, 34.0 );
        roundRect2.setRect ( 2.0, 1.5 , 196.0, 30.0 );
    }

    QPainter painter ( this );

    painter.setRenderHint ( QPainter::Antialiasing, true );
    painter.setWindow ( 0, 0, 200, 34 );

    painter.setBrush ( barBottomRgBrush );
    painter.setPen ( Qt::NoPen );
    painter.setOpacity ( 0.5 );

    //draw bottom backgraund
    painter.drawRoundRect ( rectangle1, 10, 30 );
    if ( t == RoundedTop || t == RoundedBottom )
        painter.drawRoundRect ( roundRect1, 10, 59 );


    painter.setBrush ( barTopLgBrush );
    painter.setPen ( Qt::NoPen );
    painter.setOpacity ( 1.0 );

    //draw top background
    painter.drawRoundRect ( rectangle2, 5, 5 );
    if ( t == RoundedTop || t == RoundedBottom )
        painter.drawRoundRect ( roundRect2, 10, 50 );

    //draw the ball's bottom background
    painter.setBrush ( ballBottomRgBrush );
    painter.drawEllipse ( 166, 8, 18, 18 );

    painter.setBrush ( ballTopRgBrush );
    painter.setPen ( Qt::NoPen );

    //draw the ball's bottom background
    painter.drawEllipse ( 168, 10, 14, 14 );
}
