/*
 * File: MainWidget.cpp
 * Created: Mon 31 Mar 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "MainWidget.h"

MainWidget::MainWidget::MainWidget ( QWidget *parent )
        : QWidget ( parent )
{
    setMinimumSize ( QSize ( 100, 100 ) );
    setMaximumSize ( QSize ( 600, 600 ) );
    setVisible ( false );
    op = 0.0;
    logo = false;
    logoPx.load ( ":images/logo.png" );
}

void MainWidget::paintLogo ( bool show, qreal maxOp )
{
    static bool isBeingDrawn = false;
    if ( isBeingDrawn )
        return;

    isBeingDrawn = true;
    QTimer *timer = new QTimer;
    QEventLoop el;
    connect ( timer, SIGNAL ( timeout() ), &el, SLOT ( quit() ) );
    timer->start ( 10 );

    logo = true;

    show ? op = 0.0 : op = maxOp/*1.0*/;

    while ( ( show ? op < maxOp/*1.0*/ : op > 0.0 ) )
    {
        el.exec();
        show ? op += 0.05 : op -= 0.05;
        update();
    }
    timer->stop();

    logo = show;
    isBeingDrawn = false;
    emit drawingFinished();
    //qDebug() << "finished";
}

void MainWidget::paintEvent ( QPaintEvent * )
{
    QColor frColor = QColor ( 211, 206, 195 );
    QColor bgColor = QColor ( 250, 247, 246 );
    QBrush brush ( frColor );

    QRectF rectangle1 ( 0.0, 0.0, 100.0, 100.0 );
    QRectF rectangle2 ( 0.9, 0.9, 98.2, 98.2 );

    QPainter painter ( this );

    painter.setRenderHint ( QPainter::Antialiasing, true );
    painter.setWindow ( 0, 0, 100, 100 );

    QRadialGradient rg ( QPointF ( 40, 40 ), 30 );;
    rg.setColorAt ( 0, QColor ( 194, 189, 179 ) );
    rg.setColorAt ( 1, QColor ( 173, 170, 160, 80 ) );

    QBrush gb ( rg );
    painter.setBrush ( gb );
    painter.setPen ( Qt::NoPen );

    painter.setOpacity ( 0.8 );
    painter.drawRoundRect ( rectangle1, 7, 7 );

    QLinearGradient linearGrad ( 0.0, 0.0, 0.0, 100.0 );
    linearGrad.setColorAt ( 1.0, QColor ( 242, 236, 237 ) );
    linearGrad.setColorAt ( 0.0, QColor ( 254, 253, 251 ) );

    QBrush gbrush ( linearGrad );
    painter.setBrush ( gbrush );
    painter.setPen ( Qt::NoPen );

    painter.setOpacity ( 1.0 );
    painter.drawRoundRect ( rectangle2, 5, 5 );
    //paintLogo(true);
    QRectF target ( 5, 6, 31100.0/width(), 31100.0/height() );
    QRectF source ( 0, 0, 311, 311 );

    if ( logo )
    {
        painter.setOpacity ( op );
        painter.drawPixmap ( target, logoPx, source );
    }
}
