/*
 * File: ProgressBar.cpp
 * Created: Mon 31 Mar 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "ProgressBar.h"

ProgressBar::ProgressBar ( QWidget *parent )
        : QWidget ( parent )
{
    const int a = 10;

    timer  = new QTimer;
    m_text = "";
    op1 = 0.0;
    op2 = 0.0;
    op3 = 0.0;
    op4 = 0.0;

    rectOne   = QRect ( 0, 0, a, a );
    rectTwo   = QRect ( 13, 0, a, a );
    rectThree = QRect ( 26, 0, a, a );
    rectFour  = QRect ( 38, 0, a, a );

    setVisible ( false );
}

ProgressBar::ProgressBar ( const QString &text, QWidget *parent )
        : QWidget ( parent )
{
    const int a = 10;

    timer  = new QTimer;
    m_text = text;

    op1 = 0.0;
    op2 = 0.0;
    op3 = 0.0;
    op4 = 0.0;

    rectOne   = QRect ( 76, 20, a, a );
    rectTwo   = QRect ( 88, 20, a, a );
    rectThree = QRect ( 100, 20, a, a );
    rectFour  = QRect ( 112, 20, a, a );
}

void ProgressBar::paintEvent ( QPaintEvent * )
{
    //qDebug() << "Painting bar...";
    static bool one   = true;
    static bool two   = false;
    static bool three = false;
    static bool four  = false;
    static bool end   = false;

    QPainter painter ( this );
    QBrush brush ( Qt::gray );

    painter.setRenderHint ( QPainter::Antialiasing, true );
    painter.setWindow ( 0, 0, 200, 100 );
    painter.setBrush ( brush );

    if ( !m_text.isEmpty() )
    {
        painter.setPen ( QColor ( 101, 101, 101 ) );
        QFont font ( "Free Sans", 8, QFont::Bold );
        painter.setFont ( font );
        painter.drawText ( QRect ( 0, 0, 200, 15 ), Qt::AlignHCenter | Qt::AlignTop, m_text );
    }

    painter.setPen ( Qt::NoPen );

    if ( one )
    {
        //qDebug() << "Painting one";
        if ( op1 < 1.0 )
        {
            op1 += 0.05;
            painter.setOpacity ( op1 );
        }
        else
            two = true;

        painter.drawRect ( rectOne );
    }
    if ( two )
    {
        //qDebug() << "Painting two";
        if ( op2 < 1.0 )
        {
            op2 += 0.05;
            painter.setOpacity ( op2 );
        }
        else
            three = true;

        painter.drawRect ( rectTwo );
    }
    if ( three )
    {
        //qDebug() << "Painting three";
        if ( op3 < 1.0 )
        {
            op3 += 0.05;
            painter.setOpacity ( op3 );
        }
        else
            four = true;

        painter.drawRect ( rectThree );

    }
    if ( four )
    {
        //qDebug() << "Painting four";
        if ( op3 < 1.0 )
        {
            op3 += 0.05;
            painter.setOpacity ( op4 );
        }
        else
        {
            one   = false;
            two   = false;
            three = false;
            four  = false;
            end   = true;
        }

        painter.drawRect ( rectFour );
    }

    if ( end )
    {
        //qDebug() << "Painting end";
        if ( op1 >= 0.1 )
        {
            //qDebug() << "<=1.0" << op1;
            op1 -= 0.05;
            painter.setOpacity ( op1 );

            painter.drawRect ( rectOne );
            painter.drawRect ( rectTwo );
            painter.drawRect ( rectThree );
            painter.drawRect ( rectFour );
        }
        else
        {
            //qDebug() << "else";
            op1 = 0.0;
            op2 = 0.0;
            op3 = 0.0;
            op4 = 0.0;

            one = true;
            end = false;
        }
    }
}

void ProgressBar::start()
{
    connect ( timer, SIGNAL ( timeout() ), this, SLOT ( update() ) );
    timer->start ( 50 );
}

void ProgressBar::stop()
{
    disconnect ( timer, SIGNAL ( timeout() ), this, SLOT ( update() ) );
    timer->stop();
}
