/*
 * File: UbuDSL_Cmd.cpp
 * Created: Thu 07 Feb 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU GPL v3 or newer
 */

#include "UbuDSL_Cmd.h"
#include <iostream>
#include <string>

UbuDSL_Cmd::UbuDSL_Cmd( const int lockedFD ) : m_service("")
{
    QString xmlFile = QCoreApplication::instance()->applicationDirPath();

    if (QFile::exists(xmlFile + "/configs/services.xml"))
        xmlFile += "/configs/services.xml";
    else if (QFile::exists(xmlFile + "/../configs/services.xml"))
        xmlFile += "/../configs/services.xml";
    else
        xmlFile  = ":configs/services.xml";

    sparser = new ServiceParser ( xmlFile );

    vpi = -1;
    vci = -1;
    enterLogin    = tr ( "Enter login:" );
    enterPassword = tr ( "Enter password:" );
    emptyPassword = tr ( "Password can not be empty!" );
    emptyLogin    = tr ( "Login can not be empty!" );

    const QString welcomeString = QString ( tr ( "Welcome in %1!" ) ).arg ( "UbuDSL 1.0 beta" );

    consoleWriteLine ( "#-----------------------------------------------------------------------#", false );
    consoleWriteLine ( "#-                                                                     -#", false );
    consoleWriteLine ( "#-               \t" + welcomeString + "\t               -#"                  , false );
    consoleWriteLine ( "#-                                                                     -#", false );

    consoleWriteLine ( "#-----------------------------------------------------------------------#", false );

    consoleWriteLine ( enterLogin );
    wHash();
    login = consoleReadLine();

    while ( login.isEmpty() )
    {
        consoleWriteLine ( emptyLogin );
        consoleWriteLine ( enterLogin );
        wHash();
        login = consoleReadLine();
    }

    separator();

    consoleWriteLine ( enterPassword );
    wHash();
    password = consoleReadLine();

    while ( password.isEmpty() )
    {
        consoleWriteLine ( emptyPassword );
        consoleWriteLine ( enterPassword );
        wHash();
        password = consoleReadLine();
    }


    connect ( sparser, SIGNAL ( countriesMap ( const QMap<int, QString>  & ) ), this, SLOT ( chooseCountry ( const QMap<int, QString> & ) ) );
    connect ( sparser, SIGNAL ( servicesMap ( const QMap<int, QString> & ) ), this, SLOT ( chooseService ( const QMap<int, QString> & ) ) );
    connect ( sparser, SIGNAL ( sendServiceData ( unsigned short &, unsigned short &, const QString & ) ),
              this,    SLOT ( setServiceData ( unsigned short &, unsigned short &, const QString & ) ) );

    sparser->getCountries();

    disconnect ( sparser, SIGNAL ( countriesMap ( const QMap<int, QString>  & ) ), this, SLOT ( chooseCountry ( const QMap<int, QString> & ) ) );
    disconnect ( sparser, SIGNAL ( servicesMap ( const QMap<int, QString> & ) ), this, SLOT ( chooseService ( const QMap<int, QString> & ) ) );
    disconnect ( sparser, SIGNAL ( sendServiceData ( unsigned short &, unsigned short &, const QString &, const QStringList & ) ),
                 this,    SLOT ( setServiceData ( unsigned short &, unsigned short &, const QString &, const QStringList & ) ) );

    separator();
    consoleWriteLine();

 fprintf(stderr, "%i, %i, %i\n", protocol, vpi, vci);
    thread = new UbuDSL_Thread ( login, password, protocol, vpi, vci, m_service, m_dnsServers, false, false, lockedFD );
    connect ( thread, SIGNAL ( waitingForDisc() ), this, SLOT ( setVolumesList() ) );
    connect ( thread, SIGNAL ( modemDetected ( const QString & ) ), this, SLOT ( printMessage ( const QString & ) ) );
    connect ( thread, SIGNAL ( message ( const QString & ) ), this, SLOT ( printMessage ( const QString & ) ) );
// connect( thread, SIGNAL(setVolumesList(QStringList *)), this, SLOT(setVolumesList(QStringList *)) );
    connect ( thread, SIGNAL ( finished() ), this, SIGNAL ( finished() ) );
    connect ( thread, SIGNAL ( incompatibleArch() ), this, SLOT ( incompatibleArch() ) );
    thread->start();
}

UbuDSL_Cmd::~UbuDSL_Cmd()
{
    delete thread;
}


void UbuDSL_Cmd::chooseCountry ( const QMap<int, QString> &cMap )
{
    QMapIterator<int, QString> it ( cMap );
    bool ok = false;

    separator();

    while ( it.hasNext() )
    {
        it.next();
        QString value = it.value();

        if ( it.key() < 10 )
            value.prepend ( " " );

        consoleWriteLine ( QString ( "%1. %2" ).arg ( it.key() ).arg ( value ) );
    }

    while ( !ok )
    {
        separator();
        consoleWriteLine ( tr ( "Please choose a country:" ) );
        wHash();
        int c = consoleReadLine().toInt();
        it.toFront();

        while ( it.hasNext() )
        {
            it.next();
            if ( c == it.key() )
            {
                chosenCountry = it.value();
                ok = true;
            }
        }
    }

    sparser->getServicesForCountry ( chosenCountry );
}

void UbuDSL_Cmd::chooseService ( const QMap<int, QString> &sMap )
{
    QMapIterator<int, QString> it ( sMap );
    bool ok = false;

    separator();

    while ( it.hasNext() )
    {
        it.next();
        consoleWriteLine ( QString ( "%1. %2" ).arg ( it.key() ).arg ( it.value() ) );
    }

    consoleWriteLine ( tr ( "0. Back and choose another country" ) );

    while ( !ok )
    {
        separator();
        consoleWriteLine ( tr ( "Please choose a service:" ) );
        wHash();
        int c = consoleReadLine().toInt();
        it.toFront();

        if ( c == 0 )
        {
            sparser->getCountries();
            return;
        }

        while ( it.hasNext() )
        {
            it.next();
            if ( c == it.key() )
            {
                chosenService = it.value();
                ok = true;
            }
        }
    }

    m_service = chosenService;
    sparser->requestServiceData ( chosenService );
}


void UbuDSL_Cmd::setServiceData ( unsigned short &VPI, unsigned short &VCI, const QString  &protocol, const QStringList &dnsServers )
{
    int a = 0;

    while ( a != 1 && a != 2 && a != 3 )
    {
        separator();
        consoleWriteLine ( QString ( tr ( "Chosen country:\t%1." ) ).arg ( chosenCountry ) );
        consoleWriteLine ( QString ( tr ( "Chosen service:\t%1." ) ).arg ( chosenService ) );
        consoleWriteLine ( QString ( tr ( "Service login:\t%1." ) ).arg ( login ) );
        consoleWriteLine ( QString ( tr ( "Service password:\t%1." ) ).arg ( password ) );
        consoleWriteLine();
        consoleWriteLine ( tr ( "Are the above settings correct?" ) );
        consoleWriteLine ( tr ( "1. Yes, continue." ) );
        consoleWriteLine ( tr ( "2. No, change login and/or password." ) );
        consoleWriteLine ( tr ( "3. No, go back and choose another country and/or service." ) );
        wHash();
        a = consoleReadLine().toInt();

        if ( a == 2 )
        {
            separator();
            consoleWriteLine ( tr ( "Enter login:" ) );
            wHash();
            login = consoleReadLine();
            while ( login.isEmpty() )
            {
                consoleWriteLine ( emptyLogin );
                consoleWriteLine ( enterLogin );
                wHash();
                login = consoleReadLine();
            }
            separator();
            consoleWriteLine ( tr ( "Enter password:" ) );
            wHash();
            password = consoleReadLine();
            while ( password.isEmpty() )
            {
                consoleWriteLine ( emptyPassword );
                consoleWriteLine ( enterPassword );
                wHash();
                password = consoleReadLine();
            }
            a = 0;
        }
    }

    if ( a == 3 )
    {
        sparser->getCountries();
        return;
    }

    if ( protocol == "pppoa" )
        this->protocol = 0;
    else
        this->protocol = 1;

    vpi = VPI;
    vci = VCI;
    m_dnsServers = dnsServers;
}

void UbuDSL_Cmd::consoleWriteLine ( const QString &line, bool hash )
{
    QFile *consoleOut = new QFile;

    if ( consoleOut->open ( stdout, QFile::WriteOnly ) )
    {
        if ( hash )
            consoleOut->write ( "# " );

        consoleOut->write ( line.toUtf8() );
        consoleOut->write ( "\n" );
    }
    else
    {
        qDebug() << QString ( "Could not write line (%1) to stdout!" ).arg ( line );
    }

    consoleOut->close();
    delete consoleOut;
}


void UbuDSL_Cmd::consoleWrite ( const QString &data )
{
    QFile *consoleOut = new QFile;

    if ( consoleOut->open ( stdout, QFile::WriteOnly ) )
    {
        consoleOut->write ( data.toUtf8() );
    }
    else
    {
        qDebug() << QString ( "Could not write (%1) to stdout!" ).arg ( data );
    }

    consoleOut->close();
    delete consoleOut;
}


QByteArray UbuDSL_Cmd::consoleReadLine()
{
    QFile *consoleIn = new QFile;
    QByteArray ba;

    if ( consoleIn->open ( stdin, QFile::ReadOnly ) )
    {
        ba = consoleIn->readLine ( 100 );
        ba = ba.remove ( ba.count() - 1, 1 );
    }
    else
    {
        qDebug() << "UbuDSL_Cmd::consoleReadLine(): could not open stdin!";
        return QByteArray();
    }

    return ba;
}

void UbuDSL_Cmd::printMessage ( const QString &message )
{
    consoleWriteLine ( message );
}

void UbuDSL_Cmd::setVolumesList()
{
    MediaManager *mm = new MediaManager(this->thread);
    volumes = mm->getVolumesList();
    delete mm;

    if ( volumes.count() == 0 )
    {
        consoleWriteLine ( tr ( "No volumes available." ) );
        consoleWriteLine ( tr ( "Please insert a disc into drive and press Enter to refresh the list" ) );
        wHash();
        consoleReadLine();
        setVolumesList();
        return;
    }

    consoleWriteLine();

    for ( int i = 0; i < volumes.count(); i++ )
    {
        consoleWriteLine ( QString ( "%1. %2 %3" ).arg ( i + 1 ).arg ( volumes[i].getDevice() ).arg ( volumes[i].getLabel() ) );
    }

    consoleWriteLine ( QString ( tr ( "%1. Refresh the volumes list" ) ).arg ( 0 ) );

    int idx;
    separator();
    consoleWriteLine ( tr ( "Choose a volume:" ) );
    wHash();
    idx = consoleReadLine().toInt();

    if ( idx == 0 )
    {
        setVolumesList();
        return;
    }

    idx -= 1;

    if ( idx > ( volumes.count() - 1 ) || idx < 0 )
    {
        setVolumesList();
        return;
    }

    connect ( this, SIGNAL ( volumeSelected ( const int & ) ), thread, SLOT ( volumeSelected ( const int & ) ) );
    emit volumeSelected ( idx );
    disconnect ( this, SIGNAL ( volumeSelected ( const int & ) ), thread, SLOT ( volumeSelected ( const int & ) ) );
}

void UbuDSL_Cmd::incompatibleArch()
{
    QString msg ( tr ( "Error! Your modem is incompatible with 64 bit architecture.\n"
                       "If you want to use this modem, you have to install 32 bit\n"
                       "version of your system. Aborting..." ) );

    consoleWriteLine ( msg );
    exit ( -100 );
}
