/*
 * File: UbuDSL_GUI.cpp
 * Created: Thu 07 Feb 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU GPL v3 or newer
 */

#include "UbuDSL_GUI.h"
#include "UbuDSL_Cmd.h"
#include <iostream>
#include <QtGui>
#include <QMutex>

#define VERSION "1.0 beta 1"

const QColor UbuDSL_GUI::gray_1(140, 140, 140);
const QColor UbuDSL_GUI::gray_2(213, 213, 213);
const QColor UbuDSL_GUI::orange_1(204, 124, 8);
const QColor UbuDSL_GUI::orange_2(231, 184, 87);
const QColor UbuDSL_GUI::green_1(94, 161, 0);
const QColor UbuDSL_GUI::green_2(168, 230, 79);

// void UbuDSL_GUI::mouseMoveEvent ( QMouseEvent * event )
// {
//      move(mapToGlobal(event->pos()));
//     //qDebug() << "mouse move:" << mapToParent(event->pos());
// }

void UbuDSL_GUI::paintEvent ( QPaintEvent * )
{
    QColor bgColor = QColor ( 238, 232, 220 );
    QColor gradSt  = QColor ( 208, 203, 193 );

    QRectF mainRect ( 0.0,  15.0, 600.0,   420 );
    QRectF leftGrad ( 0.0,   0.0,  15.0, 450.0 );
    QRectF rightGrad ( 585.0,   0.0,  15.0, 450.0 );
    QRectF topBar ( 0.0,   0.0, 600.0,  18.0 );
    QRectF bottomBar ( 0.0, 432.0, 600.0,  18.0 );

    QPainter painter ( this );

    painter.setRenderHint ( QPainter::Antialiasing, true );
    painter.setWindow ( 0, 0, 600, 450 );
    painter.setPen ( Qt::NoPen );

    QBrush mainBrush ( bgColor );
    painter.setBrush ( mainBrush );
    painter.drawRect ( mainRect );

    QLinearGradient linearGrad ( 0.0, 0.0, 15.0, 0.00 );
    linearGrad.setColorAt ( 0, gradSt );
    linearGrad.setColorAt ( 1, bgColor );
    QBrush lgbrush ( linearGrad );
    painter.setBrush ( lgbrush );
    painter.drawRect ( leftGrad );
    linearGrad.setStart ( 600, 0 );
    linearGrad.setFinalStop ( 585, 0 );
    QBrush lgbrush2 ( linearGrad );
    painter.setBrush ( lgbrush2 );
    painter.drawRect ( rightGrad );

    QBrush brush ( Qt::black );
    painter.setBrush ( brush );
    painter.drawRect ( topBar );
    painter.drawRect ( bottomBar );
}

Table::Table ( QWidget *parent ) : QWidget ( parent )
{
    setMinimumSize ( QSize ( 100, 100 ) );
    setMaximumSize ( QSize ( 300, 300 ) );
    lbl = new QLabel ( this );
    lbl->setGeometry ( 10, 10, 180, 190 );
    op = 0.0;
    timer = new QTimer;
    m_hidden = true;
    m_text = "";
    setVisible ( false, true );
}

void Table::paintEvent ( QPaintEvent * )
{
    QColor bgColor = QColor ( 255, 255, 255 );

    QRectF mainRect ( 0.0, 0.0, 200.0, 200.0 );

    QPainter painter ( this );
    painter.setRenderHint ( QPainter::TextAntialiasing, true );
    painter.setWindow ( 0, 0, 194, 204 );
    painter.setPen ( Qt::NoPen );

    QBrush mainBrush ( bgColor );
    painter.setOpacity ( op );
    painter.setBrush ( mainBrush );
    painter.drawRect ( mainRect );

    painter.setPen ( QColor ( 101, 101, 101 ) );
    QFont font ( "Free Sans", 8, QFont::Bold );
    font.insertSubstitutions("Free Sans", QStringList() << "Nimbus Sans L" << "Luxi Sans" << "Lucida Sans" << "Andale Sans");
    painter.setFont ( font );
    painter.drawText ( QRect ( 7, 10, 176, 184 ), Qt::AlignLeft | Qt::TextWordWrap, m_text );
}

void Table::fadeIn()
{
    if ( !m_hidden )
        return;

    if ( op >= 1.0 )
    {
        timer->stop();
        m_hidden = false;
        disconnect ( timer, SIGNAL ( timeout() ), this, SLOT ( fadeIn() ) );
        emit shown();
    }
    else
        op += 0.1;
    repaint();
}

void Table::fadeOut()
{
    if ( m_hidden )
        return;

    if ( op <= 0.0 )
    {
        timer->stop();
        m_hidden = true;
        disconnect ( timer, SIGNAL ( timeout() ), this, SLOT ( fadeOut() ) );
        emit hidden();
    }
    else
        op -= 0.1;
    repaint();
}

void Table::setVisible ( bool visible, bool instantly )
{
    if ( visible && !instantly )
    {
        op = 0.0;
        connect ( timer, SIGNAL ( timeout() ), this, SLOT ( fadeIn() ) );
        timer->start ( 30 );
    }
    else if ( !visible && !instantly )
    {
        op = 1.0;
        connect ( timer, SIGNAL ( timeout() ), this, SLOT ( fadeOut() ) );
        timer->start ( 30 );
    }
    else if ( visible && instantly )
    {
        op = 1.0;
        repaint();
    }
    else if ( !visible && instantly )
    {
        op = 0.0;
        repaint();
    }
}

void Table::setText ( const QString &t )
{
    if ( m_text == t )
        return;

    if ( !m_hidden )
    {
        hide();
        QEventLoop el;
        connect ( this, SIGNAL ( hidden() ), &el, SLOT ( quit() ) );
        el.exec();
        m_text = t;
        disconnect ( this, SIGNAL ( hidden() ), &el, SLOT ( quit() ) );
        show();
    }
    else
        m_text = t;
};

void UbuDSL_GUI::hideTable()
{
    QEventLoop el;

    if ( table->isShown() )
    {
        connect ( table, SIGNAL ( hidden() ), &el, SLOT ( quit() ) );
        table->hide();
        el.exec();
        table->move ( -200, -200 );
        disconnect ( table, SIGNAL ( hidden() ), &el, SLOT ( quit() ) );
    }
}

void UbuDSL_GUI::stepOne(int value)
{
	//qDebug() << value;
    bar4->move ( bar4->x(), value + 66 );
    bar3->move ( bar3->x(), value + 33 );
    bar2->move ( bar2->x(), value );

        //bar2->setGradient(QColor(204,124,8), QColor(231,184,87));
        //table->move ( 23, 85 );
        //table->show();
        //mw->paintLogo(true);
}

void UbuDSL_GUI::stepTwo(int value)
{
        bar2->move ( bar2->x(), value );
 
            //bar1->setGradient ( QColor ( 94, 161, 0 ), QColor ( 168, 230, 79 ) );
            //bar2->setGradient ( QColor ( 204, 124, 8 ), QColor ( 231, 184, 87 ) );


            //unLockStartButton();
            //btnBack->setVisible(true);
            //connect ( leLogin,    SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( unLockStartButton() ) );
            //connect ( lePassword, SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( unLockStartButton() ) );
            //connect ( leService,  SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( unLockStartButton() ) );
            //connect ( leCountry,  SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( unLockStartButton() ) );

            //connect ( cbCustom, SIGNAL ( clicked() ), this, SLOT ( unLockStartButton() ) );
}

void UbuDSL_GUI::stepThree(int value)
{
        bar3->move ( bar3->x(), value );
}

void UbuDSL_GUI::stepFour(int value)
{
    bar4->move ( bar4->x(), value );
}

void UbuDSL_GUI::animationFinished()
{
	if ( currentStep == 1 )
	{
		disconnect(m_timeLine, SIGNAL(frameChanged(int)), this, SLOT(stepOne(int)));
		
		bar1->setGradient ( orange_1, orange_2 );

		table->move ( 23, 85 );
		table->setText(stepOneStr);
		table->show();
		
		if (m_next)
		{
			mw->paintLogo(true);
		}  
		else
		{
			bar2->setGradient ( gray_1, gray_2 );
		}
	}
	else if ( currentStep == 2 )
	{
		disconnect(m_timeLine, SIGNAL(frameChanged(int)), this, SLOT(stepTwo(int)));
		
		bar1->setGradient ( green_1, green_2 );
		bar2->setGradient ( orange_1, orange_2 );

		mw->paintLogo(false);
                table->move ( 23, 118 );
                table->setText(stepTwoStr);
                table->show();
                confBox->show();

                btnBack->setVisible(true);
                connect ( leLogin,    SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( unLockStartButton() ) );
                connect ( lePassword, SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( unLockStartButton() ) );
                connect ( leService,  SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( unLockStartButton() ) );
                connect ( leCountry,  SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( unLockStartButton() ) );

                connect ( cbCustom, SIGNAL ( clicked() ), this, SLOT ( unLockStartButton() ) );
	}
	else if ( currentStep == 3 )
	{
		disconnect(m_timeLine, SIGNAL(frameChanged(int)), this, SLOT(stepThree(int)));
		
		bar2->setGradient ( green_1, green_2 );
		bar3->setGradient ( orange_1, orange_2 );

                confBox->hide();
                btnNext->setVisible(false);
                m_progressBar->show();
                m_progressBar->start();
                m_progressTable->show();
                table->setText ( stepThreeStr );
                table->move ( 23, 151 );
                table->show();

	        if ( cbOpenDNS->isChecked() )
	        	m_dnsServers.clear();

                if ( cbCustom->isChecked() )
                	ubuDSL_Thread = new UbuDSL_Thread ( leLogin->text(), lePassword->text(),
                  			                            comboProtocol->currentIndex(), spinVPI->value(),
                  			                            spinVCI->value(), leService->text(),
                  			                            m_dnsServers, cbAutostartAB->isChecked(),
                  			                            cbAutostartAMP->isChecked(), m_lockedFD );
                else
                  	ubuDSL_Thread = new UbuDSL_Thread ( leLogin->text(), lePassword->text(),
                  			                            comboProtocol->currentIndex(), m_VPI,
                  			                            m_VCI, comboService->currentText(), m_dnsServers,
                  			                            cbAutostartAB->isChecked(), cbAutostartAMP->isChecked(),
                  			                            m_lockedFD, m_endState );


                connect ( ubuDSL_Thread, SIGNAL ( message ( const QString & ) ), m_progressTable, SLOT ( addMessage ( const QString & ) ) );
                connect ( m_volumeSelector, SIGNAL ( volumeSelected ( const int & ) ), ubuDSL_Thread, SLOT ( volumeSelected ( const int & ) ) );
                connect ( ubuDSL_Thread, SIGNAL ( waitingForDisc() ), m_volumeSelector, SLOT ( show() ) );
//                 connect ( ubuDSL_Thread, SIGNAL ( detectingModem() ), modemWidget, SLOT ( show() ) );
                connect ( ubuDSL_Thread, SIGNAL ( modemDetected ( const QString & ) ), this, SLOT ( modemDetected ( const QString & ) ) );
                connect ( ubuDSL_Thread, SIGNAL ( finished() ), btnNext, SIGNAL ( clicked() ) );
                connect ( ubuDSL_Thread, SIGNAL ( incompatibleArch() ), this, SLOT ( incompatibleArch() ) );

                ubuDSL_Thread->start();
                this->thread()->wait(2000);
                qDebug() << "s:" << *m_endState;
	}
	else if (currentStep == 4 )
	{
		qDebug() << 222;
		disconnect(m_timeLine, SIGNAL(frameChanged(int)), this, SLOT(stepFour(int)));
		connect ( btnNext, SIGNAL ( clicked() ), this, SIGNAL ( quit() ) );

		bar3->setGradient ( green_1, green_2 );
		bar4->setGradient ( orange_1, orange_2 );

		m_progressBar->hide();
		m_progressTable->hide();
		btnNext->setText(tr("quit"));
		btnNext->show();

		m_finish = true;
		
		switch (*m_endState)
		{
			case UbuDSL_Thread::Failed:
				mainLabel->setText(stepFourErrorStr);
				break;
			case UbuDSL_Thread::Success:
				mainLabel->setText(stepFourSuccessStr);
				break;
			case UbuDSL_Thread::IncompatibleArchitecture:
				mainLabel->setText(incompatibleArchitectureStr);
				break;
		}
		
		mainLabel->setGeometry ( 10, 10, mw->width() - 20, mw->height() - 20 );
		mainLabel->show();
		mw->paintLogo(true, 0.15);
	}
	
	lockButtons(false);
	
	if (currentStep == 2)
		unLockStartButton();
}
void UbuDSL_GUI::move()
{
	qDebug() << "Move, cs:" << currentStep;
    if ( table->isShown() )
    {
        QEventLoop el;
        connect ( table, SIGNAL ( hidden() ), &el, SLOT ( quit() ) );
        table->hide();
        el.exec();
        table->move ( -200, -200 );
        disconnect ( table, SIGNAL ( hidden() ), &el, SLOT ( quit() ) );
    }
    
    if ( currentStep == 1 )
    {
	  connect(m_timeLine, SIGNAL(frameChanged(int)), this, SLOT(stepOne(int)));
	  m_timeLine->setFrameRange(83, 290);
	  m_timeLine->setDirection(QTimeLine::Forward);
		
	  lockButtons(true);
	  btnBack->setVisible(false);
	
	  m_timeLine->start();
    }
    else if ( currentStep == 2 )
    {
    	connect(m_timeLine, SIGNAL(frameChanged(int)), this, SLOT(stepTwo(int)));
    	m_timeLine->setFrameRange(290, 83);
    	m_timeLine->setDirection( m_next ? QTimeLine::Forward : QTimeLine::Backward );
    	lockButtons(true);
    	btnBack->setVisible(true);
    	
    	if (!m_next)
    	{
    		confBox->hide();
    		mw->paintLogo(true);
    	}
    	
    	m_timeLine->start();
    }
    else if ( currentStep == 3 )
    {
    	connect(m_timeLine, SIGNAL(frameChanged(int)), this, SLOT(stepThree(int)));
    	m_timeLine->setFrameRange(323, 116);
    	m_timeLine->setDirection( m_next ? QTimeLine::Forward : QTimeLine::Backward );
    	lockButtons(true);
    	btnBack->setVisible(false);
    	m_timeLine->start();
    }
    else if ( currentStep == 4 )
    {
    	connect(m_timeLine, SIGNAL(frameChanged(int)), this, SLOT(stepFour(int)));
    	m_timeLine->setFrameRange(356, 149);
    	m_timeLine->setDirection( m_next ? QTimeLine::Forward : QTimeLine::Backward );
    	lockButtons(true);
    	btnBack->setVisible(false);
    	m_timeLine->start();
    }
}

void UbuDSL_GUI::next()
{
    ++currentStep;
    m_next = true;
    //timer->start();
    move();
    //currentStep++;
}

void UbuDSL_GUI::back()
{
    --currentStep;
    m_next = false;
    //timer->start();
    //currentStep--;
    move();
}

void UbuDSL_GUI::lockButtons ( bool lock )
{
	qDebug() << "UbuDSL_GUI::lockButtons ( bool lock ):" << lock;
    btnNext->setDisabled ( lock );
    btnBack->setDisabled ( lock );
}

UbuDSL_GUI::UbuDSL_GUI ( const int lockedFD, QWidget *parent )
: QMainWindow ( parent ),
  m_lockedFD( lockedFD ),
  m_finish(false),
  m_canQuit(true),
  m_Step( 1 ),
  m_timeLine(new QTimeLine(750)),
  m_endState(new UbuDSL_Thread::EndState)
{
	this->setWindowIcon(QIcon(":images/icon.png"));
    this->setVisible ( false );
    this->setMinimumSize ( 600, 450 );
    this->setMaximumSize ( 600, 450 );
    this->setWindowTitle ( QString ( "UbuDSL %1" ).arg ( VERSION ) );
    
//     QPolygon p;
//     p << QPoint(0, 10) << QPoint(5, 5) << QPoint(10, 0) << QPoint(590, 0)
//       << QPoint(595, 5) << QPoint(600, 10) << QPoint(600, 450) << QPoint(0, 450);
//     QRegion r(p);
//     this->setMask(r);
    
    timer       = new QTimer;
    maxMoves    = 204;
    currentStep = 0;
    s_second    = false;
    s_third     = false;
    s_fourth    = false;
    mf          = false;
    m_custom    = false;
    counter     = 0;

    timer->setInterval ( 1 );
    initStrings();
    initObjects();
    initWidgets();
    initLayouts();
    initConnections();

    serviceParser->getCountries();
    
    m_timeLine->setCurveShape(QTimeLine::EaseOutCurve);
    m_timeLine->setUpdateInterval(4);

    stepZero();
}

void UbuDSL_GUI::initLayouts()
{
	qDebug() << "UbuDSL_GUI::initLayouts(): started";
    spacer1 = new QSpacerItem ( 10, 10 );
    spacer2 = new QSpacerItem ( 10, 50, QSizePolicy::Minimum, QSizePolicy::Expanding );
    spacer3 = new QSpacerItem ( 100, 50, QSizePolicy::Preferred, QSizePolicy::Minimum );

    subLay1 = new QHBoxLayout;
    subLay2 = new QHBoxLayout;
    mainLay = new QVBoxLayout ( confBox );
    mainLay->addItem ( spacer1 );
    gridLay = new QGridLayout;
    gridLay->addWidget ( labelLogin, 0, 0 );
    gridLay->addWidget ( leLogin, 0, 1 );
    gridLay->addWidget ( labelPassword, 1, 0 );
    gridLay->addWidget ( lePassword, 1, 1 );
    gridLay->addWidget ( labelCountry, 2, 0 );
    gridLay->addWidget ( comboCountry, 2, 1 );
    gridLay->addWidget ( leCountry, 2, 1 );
    gridLay->addWidget ( labelService, 3, 0 );
    gridLay->addWidget ( comboService, 3, 1 );
    gridLay->addWidget ( leService, 3, 1 );
    gridLay->addWidget ( labelProtocol, 4, 0 );
    gridLay->addWidget ( comboProtocol, 4, 1 );
    gridLay->addWidget ( labelVPI, 5, 0 );

    subLay2->addWidget ( spinVPI );
    subLay2->addWidget ( labelVCI );
    subLay2->addWidget ( spinVCI );

    gridLay->addLayout ( subLay2, 5, 1 );

    subLay1->addLayout ( gridLay );
    subLay1->addItem ( spacer3 );

    mainLay->addLayout ( subLay1 );
    //mainLay->addLayout(subLay2);
    mainLay->addWidget ( cbCustom );
    mainLay->addItem ( spacer2 );
    mainLay->addWidget ( cbOpenDNS );
    mainLay->addWidget ( cbAutostartAB );
    mainLay->addWidget ( cbAutostartAMP );

    //mainLay->setDirection(QBoxLayout::RightToLeft);
    //gridLay->setDirection(QBoxLayout::RightToLeft);
    //subLay1->setDirection(QBoxLayout::RightToLeft);
    //subLay2->setDirection(QBoxLayout::RightToLeft);
    qDebug() << "UbuDSL_GUI::initLayouts(): finished";
}

void UbuDSL_GUI::initObjects()
{
    qDebug() << "UbuDSL_GUI::initObjects(): started";

    QString system  = UbuDSL::SystemInfo::getDistributionName() + "_" + UbuDSL::SystemInfo::getDistributionRelease();
    QString xmlFile = QCoreApplication::instance()->applicationDirPath();

    if (QFile::exists(xmlFile + "/configs/services.xml"))
        xmlFile += "/configs/services.xml";
    else if (QFile::exists(xmlFile + "/../configs/services.xml"))
        xmlFile += "/../configs/services.xml";
    else
        xmlFile  = ":configs/services.xml";

    serviceParser = new ServiceParser ( xmlFile );
    //mediaManager  = new MediaManager();
    modemManager  = new ModemManager();

    qDebug() << "UbuDSL_GUI::initObjects(): finished";
}

void UbuDSL_GUI::initWidgets()
{
	qDebug() << "UbuDSL_GUI::initWidgets(): started";
    style = new UbuDSL_Style;

    bar1 = new ListBar ( ListBar::RoundedTop, this );
    bar2 = new ListBar ( ListBar::Squared, this );
    bar3 = new ListBar ( ListBar::Squared, this );
    bar4 = new ListBar ( ListBar::RoundedBottom, this );

    bar1->move ( 20, 50 );
    bar2->move ( 20, 83 );
    bar3->move ( 20, 116 );
    bar4->move ( 20, 149 );

    bar1->setGradient ( gray_1, gray_2 );
    bar2->setGradient ( gray_1, gray_2 );
    bar3->setGradient ( gray_1, gray_2 );
    bar4->setGradient ( gray_1, gray_2 );

    bar1->setText ( tr ( "introduction" ) );
    bar2->setText ( tr ( "settings" ) );
    bar3->setText ( tr ( "configuration" ) );
    bar4->setText ( tr ( "finish" ) );

    bar1->show();
    bar2->show();
    bar3->show();
    bar4->show();

    table = new Table ( this );
    table->setText ( stepOneStr );
    table->setGeometry ( -200, -200, 194, 208 );

    mw = new MainWidget ( this );
    mw->setGeometry ( 240, 50, 340, 340 );
    mw->show();

    m_progressTable = new ProgressTable ( mw );
    m_progressTable->move ( 10, 90 );

    m_progressBar = new ProgressBar ( tr("configuring"), mw );
    m_progressBar->hide();
    m_progressBar->setGeometry ( 70, 50, 200, 100 );

    confBox = new QWidget ( mw );
    confBox->setGeometry ( 0, 0, mw->width(), mw->height() );

    mainLabel = new QLabel ( mw );
    mainLabel->setWordWrap ( true );
    mainLabel->setAlignment ( Qt::AlignCenter );
    mainLabel->setGeometry ( -100, -100, 20, 20 );

    labelCountry  = new QLabel ( tr ( "Country:" ), confBox );
    labelService  = new QLabel ( tr ( "Service:" ), confBox );
    labelPassword = new QLabel ( tr ( "Password:" ), confBox );
    labelLogin    = new QLabel ( tr ( "Login:" ), confBox );
    labelVPI      = new QLabel ( "VPI:", confBox );
    labelVCI      = new QLabel ( "VCI:", confBox );
    labelProtocol = new QLabel ( tr ( "Protocol:" ), confBox );
    leLogin       = new QLineEdit ( confBox );
    lePassword    = new QLineEdit ( confBox );
    leCountry     = new QLineEdit ( confBox );
    leService     = new QLineEdit ( confBox );
    cbOpenDNS     = new QCheckBox ( tr ( "Set OpenDNS servers as default" ), confBox );
    
    
    cbAutostartAB    = new QCheckBox ( tr ( "Establish connection while booting system" ), confBox );
    cbAutostartAMP   = new QCheckBox ( tr ( "Establish connection when user plugs modem in" ), confBox );
    cbCustom         = new QCheckBox ( tr ( "Custom settings" ), confBox );

    leCountry->hide();
    leService->hide();
    labelProtocol->hide();
    labelVPI->hide();
    labelVCI->hide();

    spinVPI = new QSpinBox ( confBox );
    spinVCI = new QSpinBox ( confBox );

    spinVPI->hide();
    spinVCI->hide();

    spinVPI->setMinimum ( 0 );
    spinVPI->setMaximum ( 10 );
    spinVCI->setMinimum ( 0 );
    spinVCI->setMaximum ( 100 );

    btnBack = new QPushButton ( strBack, this );
    btnNext = new QPushButton ( strNext, this );
    btnBack->setVisible(false);
    btnNext->setGeometry ( this->width() - btnNext->width(), this->height() - 18, btnNext->width(), 18 );
    btnNext->setStyle ( style );
    btnBack->setGeometry ( 0, this->height() - 18, btnBack->width(), 18 );
    btnBack->setStyle ( style );


    comboCountry = new QComboBox ( confBox );
    comboCountry->setMaxVisibleItems ( 10 );
    comboService = new QComboBox ( confBox );
    comboService->setMaxVisibleItems ( 6 );
    comboProtocol = new QComboBox ( confBox );
    comboProtocol->hide();
    comboProtocol->insertItem ( 0, "PPPoA" );
    comboProtocol->insertItem ( 1, "PPPoE" );

    m_volumeSelector = new VolumeSelector ( this );
    //m_volumeSelector->show();

    confBox->hide();

//     modemWidget = new QWidget ( mw );
//     modemWidget->hide();
//     modemWidget->setGeometry ( 50, 220, 240, 105 );

//     labelModem = new QLabel ( tr ( "Please plug your modem in." ), modemWidget );
//     labelModem->setGeometry ( 0, 0, 230, 30 );
//     labelModem->setAlignment ( Qt::AlignHCenter );
//     labelModem->setWordWrap ( true );

//     buttonModemCancel = new QPushButton ( tr ( "cancel" ), modemWidget );
//     buttonModemCancel->move ( 230 - buttonModemCancel->width(), 100 - buttonModemCancel->height() );

    qDebug() << "UbuDSL_GUI::initWidgets(): finished";
}

void UbuDSL_GUI::initConnections()
{
	qDebug() << "UbuDSL_GUI::initConnections(): started";
    //connect ( timer         , SIGNAL ( timeout() ),
    //          this          , SLOT ( move() ) );
    connect ( btnNext          , SIGNAL ( clicked() ),
              this             , SLOT   ( next() ) );
    connect ( btnBack          , SIGNAL ( clicked() ),
              this             , SLOT   ( back() ) );
    connect ( modemManager     , SIGNAL ( foundModem() ),
              this             , SLOT   ( foundModem() ) );

    connect ( serviceParser    , SIGNAL ( foundCountry ( const QString & ) ),
              this             , SLOT   ( addCountry ( const QString & ) ) );
    connect ( serviceParser    , SIGNAL ( foundService ( const QString & ) ),
              this             , SLOT   ( addService ( const QString & ) ) );

    connect ( serviceParser    , SIGNAL ( sendServiceData ( const int &,
                                          const int &,
                                          const QString  &, const QStringList & ) ),
              this             , SLOT   ( setServiceData ( const int &,  const int &,
                                                      const QString  &, const QStringList & ) ) );
    connect ( comboCountry     , SIGNAL ( currentIndexChanged ( const QString & ) ),
              comboService     , SLOT   ( clear() ) );
    connect ( comboCountry     , SIGNAL ( currentIndexChanged ( const QString & ) ),
              serviceParser    , SLOT   ( getServicesForCountry ( const QString & ) ) );
    connect ( comboService     , SIGNAL ( currentIndexChanged ( const QString & ) ),
              serviceParser    , SLOT   ( requestServiceData ( const QString & ) ) );
    connect ( cbCustom         , SIGNAL ( toggled ( bool ) ),
              this             , SLOT   ( cbCustomToggled ( bool ) ) );
    connect ( m_volumeSelector , SIGNAL ( quit() ),
              this             , SIGNAL ( quit() ) );
    connect ( m_timeLine       , SIGNAL ( finished() ),
    		  this         , SLOT   ( animationFinished() ) );
// connect( mediaManager  , SIGNAL(volumeAdded(const QString &, const QString &, const QString &)),
//          this          , SLOT(volumeAdded(const QString &, const QString &, const QString &)) );
// connect( mediaManager  , SIGNAL(volumeRemoved(const QString &)),
//          this          , SLOT(volumeRemoved(const QString &)) );

    qDebug() << "UbuDSL_GUI::initConnections(): finished";
}

void UbuDSL_GUI::initStrings()
{
    QString email = "<b>ubudsl@gmail.com</b>";

    stepOneStr         = tr ( "Welcome to UbuDSL - a tool that helps you configure your "
                              "ADSL connection on Linux Operating System.\n\nBefore start "
                              "please unplug your modem from PC and press the \"next\" "
                              "button to continue." );
    stepTwoStr         = tr ( "All you have to do is choose your country and service from "
                              "the list on right, type the service login and password and "
                              "click \"next\" to start configuration.\n\nIt is possible "
                              "that you will have to insert your system installation disc "
                              "into the drive. Please do it, when UbuDSL asks your for it." );
    stepThreeStr       = tr ( "Please be patient. The configuration may take from few "
                              "seconds to several minutes - it depends on your system "
                              "and your modem." );
    stepFourSuccessStr = tr ( "Configuration finished successfully. Press \"quit\" to "
                              "exit application. It is strongly recommended to reboot "
                              "your computer now because some modems need this.\n\n"
                              "You can easily manage your connection via Connection "
                              "Monitor provided with UbuDSL." );
    stepFourErrorStr  = QString ( tr ( "Configuration finished, however, there have occured some "
                                       "errors and it is possible that establishing connection "
                                       "may not be possible. In such case please generate a log (you "
                                       "can do this via the Connection Monitor applet's menu) and "
                                       "contact us at %1. Do not forget to attach the log!" ) ).arg ( email );
    incompatibleArchitectureStr = tr ( "Configuration has been aborted because of an incompatible "
    		  						   "architecture - modem's manufacturer does not provide any "
    		                           "64-bit compatible driver. You have to install the 32-bit "
    		                           "version of your system. We are sorry...");
    
    strBack = tr ( "back" );
    strNext = tr ( "next" );
}
void UbuDSL_GUI::stepZero()
{
    QTimer *tmpTimer = new QTimer;
    QEventLoop *el = new QEventLoop;

    connect ( tmpTimer, SIGNAL ( timeout() ), el, SLOT ( quit() ) );

    tmpTimer->setSingleShot ( true );
    tmpTimer->start ( 500 );
    el->exec();

    emit next();

    delete tmpTimer;
    delete el;
}

void UbuDSL_GUI::cbCustomToggled ( bool checked )
{
    comboCountry->setVisible ( !checked );
    leCountry->setVisible ( checked );
    comboService->setVisible ( !checked );
    leService->setVisible ( checked );

    labelVPI->setVisible ( checked );
    labelVCI->setVisible ( checked );
    spinVPI->setVisible ( checked );
    spinVCI->setVisible ( checked );

    labelProtocol->setVisible ( checked );
    comboProtocol->setVisible ( checked );

    m_custom = checked;
}

UbuDSL_GUI::~UbuDSL_GUI()
{
	qDebug() << "UbuDSL_GUI::~UbuDSL_GUI(): started";
	
	if (ubuDSL_Thread->isRunning())
	{
		ubuDSL_Thread->terminate();
	}
	
    delete ubuDSL_Thread;
    qDebug() << "UbuDSL_GUI::~UbuDSL_GUI(): finished";
}


void UbuDSL_GUI::foundModem()
{
    mf = true;
}


void UbuDSL_GUI::addCountry ( const QString &country )
{
    comboCountry->addItem ( country );
}


void UbuDSL_GUI::addService ( const QString &service )
{
    qDebug() << service;
    comboService->addItem ( service );
}


void UbuDSL_GUI::setServiceData ( const int &VPI, const int &VCI, const QString  &protocol, const QStringList &dnsServers )
{
	printf("vci set to: %i\n", VCI);

    m_dnsServers = dnsServers;
	
    spinVPI->setValue( VPI );
    spinVCI->setValue( VCI );
    
    m_VPI = VPI;
    m_VCI = VCI;

    if ( protocol == "pppoa" )
        comboProtocol->setCurrentIndex ( 0 );
    else
        comboProtocol->setCurrentIndex ( 1 );
}


/*void UbuDSL_GUI::on_startButton_clicked()
{
 //m_progressBar->show();


 modemManager->connectToDBus();
 while ( !mf )
 {
  qDebug() << "Detecting modem";
  QTest::qWait( 5000 );
 }

 qDebug() << QString("Modem name: %1, driver name: %2, driver version: %3, needs firmware:")
                    .arg(modemManager->getModemName())
                    .arg(modemManager->getDriverName())
                    .arg(modemManager->getDriverVersion()) << modemManager->getDoesModemNeedFirmware()
                    << modemManager->getIsDriverInSystem();

 if ( !modemManager->getIsDriverInSystem() )
 {
  Compiler *compiler = new Compiler("test.ini");
  compiler->removeDriver();
  compiler->extractDriver();
  qDebug() << "czekam na entera";
  std::cin.get();
  compiler->compileDriver();
  compiler->installDriver();
  delete compiler;
 }

 QString login      = loginLine->text();
 QString password   = passwordLine->text();
 unsigned short VPI = vpiSpin->value();
 unsigned short VCI = vciSpin->value();
 QString protocol;

 if ( comboProtocol->currentIndex() == 0 )
     protocol = "pppoa";
 else
     protocol = "pppoe";

 ubuDSL_Thread = new UbuDSL_Thread( QString(), QString(), login, password, comboProtocol->currentIndex(), VPI, VCI );

 connect( ubuDSL_Thread, SIGNAL(message(const QString)), this, SLOT(appendMessage(const QString)) );
 connect( ubuDSL_Thread, SIGNAL(finished()), this, SLOT(threadFinished()) );

 ubuDSL_Thread->start();
}*/


void UbuDSL_GUI::threadFinished()
{
    //m_progressBar->hide();
}

void UbuDSL_GUI::msg ( const QString &msg )
{
    qDebug() << msg;
}

void UbuDSL_GUI::modemDetected ( const QString &name )
{
//     buttonModemCancel->setEnabled ( false );
//     labelModem->setText ( QString ( tr ( "Modem found:\n%1." ) ).arg ( name ) );
    m_progressBar->setText( name );

//     QTimer::singleShot ( 3000, modemWidget, SLOT ( hide() ) );
}

void UbuDSL_GUI::unLockStartButton()
{
    bool unlock = true;

    if ( leLogin->text().isEmpty() )
        unlock = false;

    if ( lePassword->text().isEmpty() )
        unlock = false;

    if ( m_custom )
    {
        if ( leCountry->text().isEmpty() )
            unlock = false;

        if ( leService->text().isEmpty() )
            unlock = false;
    }

    btnNext->setEnabled ( unlock );
}

void UbuDSL_GUI::incompatibleArch()
{
	next();
    /*QString title ( tr ( "Incompatible architecture" ) );
    QString text  ( tr ( "Your modem is incompatible with 64 bit architecture. "
                         "If you want to use this modem, you have to install 32 bit "
                         "version of your system. Aborting..." ) );

    QMessageBox::critical ( this, title, text );
    exit ( -100 );*/
}

void UbuDSL_GUI::closeEvent( QCloseEvent *event )
{
	if (!m_canQuit || !ubuDSL_Thread->getCanExit())
	{
		QMessageBox::information(this, tr("Can not quit!"), tr("The application can't quit now because there are some improtant system actions runing..."));
		event->ignore();
	}
	else if (m_finish)
	{
		event->accept();
	}
	else
	{
		QMessageBox msgBox(QMessageBox::Question, tr("Do you want to quit?"),
				           tr("Do you really want to quit the application?"),
				           QMessageBox::No, this);
		QPushButton *confirmBtn = msgBox.addButton(tr("&Yes"), QMessageBox::YesRole);
		msgBox.exec();
		if (msgBox.clickedButton() == confirmBtn)
		{
			event->accept();
		}
	}
}
