/*
 * File: UbuDSL_GUI.h
 * Created: Thu 07 Feb 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU GPL v3 or newer
 */

#ifndef UBUDSLGUI_H
#define UBUDSLGUI_H

#include <QtGui>
#include "libubudsl/libubudsl.h"
#include "UbuDSL_Thread.h"
#include "ListBar.h"
#include "MainWidget.h"
#include "ProgressBar.h"
#include "VolumeSelector.h"
#include "ProgressTable.h"
#include <UbuDSL_Style.h>

class Table : public QWidget
{
    Q_OBJECT
public:
    Table ( QWidget *parent = 0 );
    qreal getOpacity()
    {
        return op;
    }
    bool isHidden()
    {
        return m_hidden;
    }
    bool isShown()
    {
        return !m_hidden;
    }
    QString text()
    {
        return m_text;
    }
public slots:
    void move ( int x, int y )
    {
        QWidget::move ( x, y );
    }
    void setVisible ( bool visible, bool instantly = false );
    void show()
    {
        setVisible ( true );
    }
    void hide()
    {
        setVisible ( false );
    }
    void setText ( const QString &t );
protected:
    void paintEvent ( QPaintEvent *event );
private slots:
    void fadeIn();
    void fadeOut();
private:
    qreal op;
    QTimer *timer;
    bool m_hidden;
    QLabel *lbl;
    QString m_text;
signals:
    void hidden();
    void shown();
};


class UbuDSL_GUI: public QMainWindow
{
    Q_OBJECT
public:
    UbuDSL_GUI ( const int lockedFD, QWidget *parent = 0 );
    ~UbuDSL_GUI();

public slots:
    void msg ( const QString &msg );
    void move ( int x, int y ) { QMainWindow::move ( x, y ); }
    void move ( const QPoint &p ) { QMainWindow::move ( p ); }
private slots:
    void threadFinished();
    void setServiceData ( const int &VPI, const int &VCI, const QString  &protocol, const QStringList &dnsServers );
    void addCountry ( const QString &country );
    void addService ( const QString &service );
    //void updateVolumesList();
    //void volumeAdded(const QString &blockDev, const QString &label, const QString &device);
    //void volumeRemoved(const QString &device);
    void foundModem();
    void cbCustomToggled ( bool checked );

    void move();
    void next();
    void back();
    void modemDetected ( const QString &name );
    void unLockStartButton();
    void incompatibleArch();
    void stepOne(int value);
    void stepTwo(int value);
    void stepThree(int value);
    void stepFour(int value);
    void animationFinished();

private:
    const int m_lockedFD;
    UbuDSL_Thread *ubuDSL_Thread;
    ServiceParser *serviceParser;
    MediaManager *mediaManager;

    QPixmap pixmap, px;
    QPainter *p;
    QLabel *l;
    UbuDSL_Style *style;

    //QStringList *volumesList;
    int volumesCount;
    ModemManager *modemManager;
    bool mf;

    ListBar *bar1;
    ListBar *bar2;
    ListBar *bar3;
    ListBar *bar4;
    Table *table;
    QTimer *timer;
    QPushButton *btnBack;
    QPushButton *btnNext;
    QString strBack;
    QString strNext;
    QComboBox *comboCountry;
    QComboBox *comboService;
    QComboBox *comboProtocol;
    QLineEdit *leLogin;
    QLineEdit *lePassword;
    QLineEdit *leCountry;
    QLineEdit *leService;
    bool m_custom;
    bool m_next;
    int maxMoves;
    int currentStep;
    void stepZero();
    
    void hideTable();
    void initStrings();
    void initWidgets();
    void initConnections();
    void initObjects();
    void initLayouts();
    virtual void closeEvent( QCloseEvent *event );

    void lockButtons ( bool lock = false );

    bool s_fourth; //steps
    bool s_third;
    bool s_second;
    bool m_finish;
    bool m_canQuit;
    int counter;

    QString stepOneStr;
    QString stepTwoStr;
    QString stepThreeStr;
    QString stepFourSuccessStr;
    QString stepFourErrorStr;
    QString incompatibleArchitectureStr;

    QStringList m_dnsServers;

    MainWidget *mw;
    QLabel *mainLabel;
    QLabel *labelLogin;
    QLabel *labelPassword;
    QLabel *labelCountry;
    QLabel *labelService;
    QLabel *labelVPI;
    QLabel *labelVCI;
    QLabel *labelProtocol;

    QSpinBox *spinVPI;
    QSpinBox *spinVCI;

    QGridLayout *gridLay;
    QVBoxLayout *mainLay;
    QHBoxLayout *subLay1;
    QHBoxLayout *subLay2;
    QSpacerItem *spacer1;
    QSpacerItem *spacer2;
    QSpacerItem *spacer3;

    QCheckBox *cbOpenDNS;
    QCheckBox *cbAutostartAB;
    QCheckBox *cbAutostartAMP;
    QCheckBox *cbCustom;

    int m_VPI;
    int m_VCI;
    
    ProgressTable *m_progressTable;
    QWidget *confBox;
//     QWidget *modemWidget;
//     QLabel  *labelModem;
//     QPushButton *buttonModemCancel;
    ProgressBar    *m_progressBar;
    VolumeSelector *m_volumeSelector;
    QMap<QString, QString> volumesMap;
    
    const int m_Step;
    
    QTimeLine *m_timeLine;
    UbuDSL_Thread::EndState *m_endState;

protected:
    void paintEvent ( QPaintEvent *event );
//     void mouseMoveEvent ( QMouseEvent * event );
    static const QColor gray_1;
    static const QColor gray_2;
    static const QColor orange_1;
    static const QColor orange_2;
    static const QColor green_1;
    static const QColor green_2;
signals:
    void quit();
};

#endif
