/*
 * File: UbuDSL_Style.cpp
 * Created: Mon 31 Mar 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "UbuDSL_Style.h"

void UbuDSL_Style::drawComplexControl ( ComplexControl control, const QStyleOptionComplex *option,
                                        QPainter *painter, const QWidget *widget ) const
{
    switch ( control )
    {
    case ( QStyle::CC_ComboBox ) :
                    if ( const QStyleOptionComboBox *comboOption = qstyleoption_cast<const QStyleOptionComboBox *> ( option ) )
            {
                QRect arrowRect = QCommonStyle::subControlRect ( CC_ComboBox, comboOption, SC_ComboBoxArrow, widget );
                QRect frameRect = QCommonStyle::subControlRect ( CC_ComboBox, comboOption, SC_ComboBoxFrame, widget );
                QRect mainRect  = frameRect;

                mainRect.setWidth ( mainRect.width() - 2 );
                mainRect.setHeight ( mainRect.height() - 2 );
                mainRect.setX ( mainRect.x() + 1 );
                mainRect.setY ( mainRect.y() + 1 );

                QColor frColor ( 211, 206, 195 );
                QColor frAColor ( 211, 206, 195, 150 );
                QColor bgColor ( 250, 244, 231 );

                QRadialGradient rg ( QPoint ( frameRect.width() / 2, frameRect.width() ), frameRect.width() );

                rg.setColorAt ( 0, frColor );
                rg.setColorAt ( 1, frAColor );

                QBrush brush ( rg );
                QBrush brush2 ( bgColor );

                painter->setPen ( Qt::NoPen );
                painter->setBrush ( brush );
                painter->setRenderHint ( QPainter::Antialiasing, true ) ;

                painter->drawRoundRect ( frameRect, 10, 60 );

                painter->setPen ( Qt::NoPen );
                painter->setBrush ( brush2 );

                painter->drawRoundRect ( mainRect, 10, 60 );

                painter->setPen ( frColor );

                QLine line ( arrowRect.x() - 1, arrowRect.y() + 1, arrowRect.x() - 1, arrowRect.height() );

                painter->drawLine ( line );
            }
        break;

    default:
        QCommonStyle::drawComplexControl ( control, option, painter, widget );
    }
}

void UbuDSL_Style::drawControl ( ControlElement control, const QStyleOption *option,
                                 QPainter *painter, const QWidget *widget ) const
{
    switch ( control )
    {
    case ( QStyle::CE_PushButton ) :
                    if ( const QStyleOptionButton *buttonOption = qstyleoption_cast<const QStyleOptionButton *> ( option ) )
            {
                QRect mainRect = buttonOption->rect/*QCommonStyle::subElementRect( SE_PushButtonFocusRect, buttonOption, widget )*/;

                QColor bgColor ( Qt::black );

                int x = ( mainRect.width() - mainRect.x() ) / 2;
                int y = ( mainRect.height() - mainRect.y() ) / 2;
                QRadialGradient rg ( x, y, x );

                rg.setColorAt ( 0.0, QColor ( 255, 255, 255, 100 ) );
                rg.setColorAt ( 0.5, QColor ( 255, 255, 255, 100 ) );
                rg.setColorAt ( 1.0, QColor ( 255, 255, 255, 0 ) );

                QBrush brush ( rg );
                QBrush brush2 ( bgColor );

                painter->setPen ( Qt::NoPen );
                painter->setBrush ( brush2 );
                painter->setRenderHint ( QPainter::Antialiasing, true ) ;


                painter->drawRect ( mainRect );
                if ( buttonOption->state == 73989 )
                {
                    //qDebug() << "\n\nstate:" << buttonOption->state;
                    painter->setBrush ( brush );
                    painter->drawRect ( mainRect );
                }
                if ( widget->isEnabled() )
                    painter->setPen ( Qt::white );
                else
                    painter->setPen ( Qt::gray );

                QFont font;
                font.setStyleHint ( QFont::SansSerif );
                font.setBold ( true );
                font.setPixelSize ( 13 );

                painter->setFont ( font );
                painter->drawText ( mainRect, Qt::AlignCenter, buttonOption->text );
            }
        break;

    default:
        QCommonStyle::drawControl ( control, option, painter, widget );
    }
}
