/*
 * File: UbuDSL_Thread.h
 * Created: Thu 07 Feb 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU GPL v3 or newer
 */

#ifndef UBUDSLTHREAD_H
#define UBUDSLTHREAD_H

#include "libubudsl/libubudsl.h"
#include <QtCore>


class UbuDSL_Thread: public QThread
{
    Q_OBJECT
public:
	enum EndState
	{
		Success = 1,
		Failed = 2,
		IncompatibleArchitecture = 3
	};
	
	UbuDSL_Thread( const QString &login, const QString &password, const int &protocol,
	                const int &VPI, const int &VCI, const QString &service,
	                const QStringList &dnsServers, const bool &connectAtBoot = false,
	                const bool &connectAtModemPlug = false, const int lockedFD = 0,
                        EndState *endState = 0
                  );

    ~UbuDSL_Thread();
    bool getCanExit();

public slots:
    void run();
    void volumeSelected ( const int &index );

private:
    void initDriversMap();
    FileManager    *fileManager;
    PackageManager *packageManager;
    ScriptManager  *scriptManager;
    ModemManager   *modemManager;
    MediaManager   *mediaManager;
    const int       m_lockedFD;
    QString         login;
    QString         password;
    int             VPI;
    int             VCI;
    int             protocol;
    QString         modemsXmlFile;
    QString         osXmlFile;

    bool            driverInSystem;
    QString         modemName;
    QString         driverName;
    QString         driverVersion;
    int             modemID;
    int             modemVendor;

    QList<Volume>   volumes;
    QString         volumeMountPoint;
    int             volumesCount;

    QSettings      *daemonSettings;
    QSettings      *systemIni;

    QStringList     packagesList;
    QStringList     packagesToInstall;
    QStringList     cdIndex;

    QString         m_sys;
    QString         m_ver;
    QString         m_driver;
    QString         m_modprobeOptions;

    QStringList     m_dnsServers;

    QList<Compiler::ModemDriver> m_drivers;
    
    bool m_canExit;
    EndState *m_endState;

    void waitForDisc();

private slots:
    void volumesListChanged();

signals:
    void message ( const QString &msg );
    void detectingModem ();
    void modemDetected ( const QString &name );
    void waitingForDisc ();
    void setVolumesList ( QList<Volume> );
    void discMounted ();
    void packageNotFound ( const QString &pkg );
    void incompatibleArch ();
};

#endif
