/*
 * File: VolumeSelector.h
 * Created: Mon 31 Mar 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU GPL v3 or newer
 */

#ifndef VOLUMESELECTOR_H
#define VOLUMESELECTOR_H

#include <QtGui/QDialog>
#include <QList>
#include <QListIterator>
#include <QListWidget>
#include <QListWidgetItem>
#include <QDebug>
#include <QThread>
#include <QCloseEvent>
#include <QMessageBox>
#include <QAbstractButton>

#include "ui_VolumeSelector.h"
#include "libubudsl/libubudsl.h"

class EjectThread: public QThread
{
    Q_OBJECT
public:
    EjectThread();
    ~EjectThread();

    void setVolume ( const QString &udi ) { m_volume = udi; }
    
public slots:
    void run();
    
private:
    QString m_volume;
    MediaManager *m_mediaManager;
    
signals:
    void volumeEjected ( const QString &volume );
};

class VolumeSelector : public QDialog, public Ui::VolumeSelector
{
    Q_OBJECT

public:
    VolumeSelector ( QWidget *parent = 0 );
    ~VolumeSelector();

private slots:
    void addVolume ( const QString &blockDev, QString label, const QString &udi );
    void removeVolume ( const QString &udi );
    void updateButtons();
    void volumeSelected();
    void ejectVolume();
    void removeFromEjectingList ( const QString &volume );
    void cancel();
protected:
    virtual void closeEvent ( QCloseEvent *e );
private:
    Ui::VolumeSelector ui;
    MediaManager *mediaManager;
    EjectThread *eject;
    QStringList volumesBeingEjected;
    bool m_quit;
signals:
    void volumeSelected ( const int &index );
    void quit();
};

#endif // VOLUMESELECTOR_H
