#include "ConnectThread.h"

//extern bool  debug;
//extern FILE *pFile;


ConnectThread::ConnectThread(QSettings *settings, int (*fGetModemState)())
: pppd(new MyProcess(0)),
  getModemState(fGetModemState),
  m_env(QProcess::systemEnvironment()),
  m_settings(settings)
{
	bool ldLibMet = false;
	bool pathMet  = false;
	
	for ( int i = 0; i < m_env.size(); i++ )
	{
		if ( m_env[i].startsWith( "LD_LIBRARY_PATH=" ) )
		{
			ldLibMet = true;
			m_env[i] = m_env[i].append( ":/opt/ubudsl/lib" );
		}
		else if ( m_env[i].startsWith( "PATH=" ) )
		{
			pathMet = true;
			m_env[i] = m_env[i].append( ":/opt/ubudsl/bin" );
		}
	}
	
	if ( !ldLibMet )
		m_env.append( "/opt/ubudsl/lib" );
	if ( !pathMet )
		m_env.append( "/opt/ubudsl/bin" );
	
	pppd->setEnvironment(m_env);
	pppd->moveToThread(this);
	connect(pppd, SIGNAL(lineAvailable(QByteArray)), this, SIGNAL(debugMsg(QByteArray)));
}

ConnectThread::~ConnectThread()
{
}

void ConnectThread::run()
{
	m_returnState = ReturnState::Unknown;
	
	printf("ConnectThread::run()\n");
	
	if ( !waitForSynchronization() )
	{
		emit threadMessage( "ConnectThread::run(): modem synchronization failed, returning!" );
		m_returnState = ReturnState::SynchronizationFailed;
		return;
	}
	
	m_settings->sync();
	
	m_arguments.clear();
	m_arguments << "debug"
	            << "linkname"
	            << "ubudsl"
		        << "call";

    if ( m_settings->value("vendor_id", 0).toInt() == 0x0483 )
    {
    	m_arguments << "ubudsl_unicorn";
    	prepareDSL0();
    }
    else if ( m_settings->value("protocol", "PPPoA").toString() == "PPPoE" )
    {
    	m_arguments << "ubudsl";
    	prepareBridging();
    	prepareNAS0();
    }
    else
    {
    	m_arguments << "ubudsl";
    }

	if ( !m_settings->value("pppd_arguments", QStringList()).toStringList().isEmpty() )
		m_arguments += m_settings->value("pppd_arguments").toStringList();
	

	pppd->start("pppd", m_arguments);
	pppd->waitForFinished(90000);
}

void ConnectThread::prepareNAS0()
{
	QNetworkInterface nas = QNetworkInterface::interfaceFromName("nas0");
	if ( nas.isValid() )
	{
		if ( nas.flags().testFlag(QNetworkInterface::IsUp) )
		{
			emit threadMessage("ConnectThread::prepareNAS0(): interface nas0 is up.");
		}
		else
		{
			emit threadMessage("ConnectThread::prepareNAS0(): interface nas0 is down.");
			QProcess ifconfig;
			ifconfig.start("ifconfig", QStringList() << "nas0" << "up");
			ifconfig.waitForFinished(30000);
			
			if ( ifconfig.exitCode() == 0 )
				emit threadMessage("ConnectThread::prepareNAS0(): ifconfig nas0 up: success.");
			else
				emit threadMessage("ConnectThread::prepareNAS0(): ifconfig nas0 up: failed!");
		}
	}
}

void ConnectThread::prepareDSL0()
{
	QNetworkInterface dsl = QNetworkInterface::interfaceFromName("dsl0");
	
	if ( dsl.isValid() )
	{
		if ( dsl.flags().testFlag(QNetworkInterface::IsUp) )
		{
			emit threadMessage("ConnectThread::prepareDSL0(): interface dsl0 is up.");
		}
		else
		{
			emit threadMessage("ConnectThread::prepareDSL0(): interface dsl0 is down.");
			QProcess ifconfig;
			ifconfig.moveToThread(this);
			ifconfig.start("ifconfig", QStringList() << "dsl0" << "up");
			ifconfig.waitForFinished(30000);
			
			if ( ifconfig.exitCode() == 0 )
				emit threadMessage("ConnectThread::prepareDSL0(): ifconfig dsl0 up: success.");
			else
				emit threadMessage("ConnectThread::prepareDSL0(): ifconfig dsl0 up: failed!");
		}
	}
}

void ConnectThread::prepareBridging()
{
	QProcess proc;
	proc.moveToThread(this);
	proc.setEnvironment(m_env);
	proc.start("modprobe", QStringList() << "br2684");
	proc.waitForFinished(5000);
	
	const QString VpiVci = m_settings->value("vpi", 0).toString() + "." + m_settings->value("vci", 0).toString();
	emit threadMessage("ConnectThread::prepareBridging(): VPI.VCI: " + VpiVci);

	proc.start( "/opt/ubudsl/bin/br2684ctl", QStringList() << "-c" << "0" << "-a" << VpiVci);
	proc.waitForFinished(5000);
	qDebug() << proc.pid();
	
	if ( proc.exitCode() == 0 )
		emit threadMessage("ConnectThread::prepareBridging(): br2684ctl: success.");
	else
		emit threadMessage("ConnectThread::prepareBridging(): br2684ctl: failed!");
	
	QTimer::singleShot(1500, this, SLOT(quit()));
	exec();
	
	//proc.start("ip", QStringList() << "addr" << "add" << "192.168.0.1/32" << "dev" << "nas0" );
	proc.start("ifconfig", QStringList() << "nas0" << "192.168.2.1" <<  "netmask" << "255.255.255.0" );
	proc.waitForFinished(5000);
	
	if ( proc.exitCode() == 0 )
		emit threadMessage("ConnectThread::prepareBridging(): ifconfig: success.");
	else
		emit threadMessage("ConnectThread::prepareBridging(): ifconfig: failed!");

	QTimer::singleShot(1500, this, SLOT(quit()));
	exec();
}

bool ConnectThread::waitForSynchronization()
{
	static int counter = 0;
	
	printf("wFS(): counter == %i\n", counter);
	
	if ( getModemState() == Synchronized )
	{
		printf("getModemState called\n");
		if ( debug )
			fprintf( pFile, "ConnectThread::waitForSynchronization(): modem synchronized.\n" );
		
		counter = 0;
		return true;
	}
	
	if ( counter > 6 )
	{
		if ( debug )
			fprintf( pFile, "ConnectThread::waitForSynchronization(): counter > 6, resetting and returning.\n" );
		
		counter = 0;
		m_returnState = ReturnState::SynchronizationFailed;
		
		return false;
	}
	
	if ( counter == 0 )
	{
		if ( debug )
			fprintf( pFile, "ConnectThread::waitForSynchronization(): counter == 0, emitting the synchronizing signal.\n" );
		
		emit synchronizing();
	}
	
	QEventLoop el(0);
	QTimer::singleShot( 5000, &el, SLOT(quit()) );
	el.exec();
	
	counter++;
	
	return waitForSynchronization();
}

ReturnState::ReturnState ConnectThread::getReturnState()
{
	return m_returnState;
}

void ConnectThread::setModemState(int state)
{
	m_modemState = state;
}
