#include "UbuDSL_Daemon.h"
/*
QDBusInterface halManager_iface( "org.freedesktop.Hal",
								 "/org/freedesktop/Hal/Manager",
								 "org.freedesktop.Hal.Manager",
								 QDBusConnection::systemBus() );

QDBusInterface halDevice_iface( "org.freedesktop.Hal",
								"/org/freedesktop/Hal/Device",
								"org.freedesktop.Hal.Device",
								QDBusConnection::systemBus() );
*/

//unicorn_msw: Modem Init Failure: Spurious ATU Detected// TODO: implement


QSettings *UbuDSL_Daemon::m_settings = new QSettings("/opt/ubudsl/.daemon_ng.ini", QSettings::IniFormat);
QFile     *UbuDSL_Daemon::m_logFile  = new QFile("/opt/ubudsl/daemon.log");

QMutex UbuDSL_Daemon::m_mutexLog;

UbuDSL_Daemon::UbuDSL_Daemon(QObject* parent)
	: QObject( parent ),
  	  dbus( QDBusConnection::systemBus() ),
	  m_justRun(true),
	  m_connecting( false ),
	  m_compiling( false ),
	  m_halManager( new QDBusInterface("org.freedesktop.Hal", "/org/freedesktop/Hal/Manager", "org.freedesktop.Hal.Manager", QDBusConnection::systemBus()) ),
//	  m_halDevice(  new QDBusInterface("org.freedesktop.Hal", "/org/freedesktop/Hal/Device" , "org.freedesktop.Hal.Device" , QDBusConnection::systemBus()) ),
	  m_IP( "..." ),
	  m_lastUDI( "#" ),
	  m_newUDI( "#" ),
	  m_deviceTimer( new QTimer() ),
	  m_timer( new QTimer() ),
	  m_rx_bytes( 0 ),
	  m_tx_bytes( 0 ),
	  m_compilerThread( new CompilerThread(*this) )
{
	if (m_logFile->exists())
    	m_logFile->remove();

    writeLog( "UbudslDaemon::UbudslDaemon(): ctor invoked." );
    
    QStringList daemonOptions = m_settings->allKeys();
    
    foreach ( QString key, daemonOptions )
    {
        writeLog( QString( "UbudslDaemon::UbudslDaemon(): daemon.ini: %1 = %2" )
        		 .arg( key, m_settings->value( key ).toString() ) );
    }
    
    new DaemonAdaptor(this);

    if (QFile::exists("/etc/dbus-1/system.d/ubudsl.conf"))
        writeLog("UbudslDaemon::UbudslDaemon(): ubudsl.conf exists.");
    else
    {
        writeLog("UbudslDaemon::UbudslDaemon(): ubudsl.conf does not exist! Creating a new one...");
    }


    if (dbus.isConnected())
        writeLog("UbudslDaemon::UbudslDaemon(): successfully connected to DBus.");
    else
        writeLog("UbudslDaemon::UbudslDaemon(): failed to connect to DBus!");

    
    if (!dbus.registerService("com.ubudsl.Daemon"))
    {
    	QString lastError = dbus.lastError().message();
    	
        writeLog( "UbudslDaemon::UbudslDaemon(): failed to register the service!" );
        writeLog( "UbudslDaemon::UbudslDaemon(): " + lastError );
        
        if ( !debug )
        	printf( "UbudslDaemon::UbudslDaemon(): Failed to register the service!\n" );
        
        exit(1);
    }
    
    if (!dbus.registerObject("/com/ubudsl/Daemon", this))
    {
        QString lastError = dbus.lastError().message();
        
        writeLog( "UbudslDaemon::UbudslDaemon(): failed to register the object!" );
        writeLog( "UbudslDaemon::UbudslDaemon(): " + lastError );
        
        if ( !debug )
        	fprintf( stderr, "Failed to register the object!\n" );
        
        exit(2);
    }
    
    m_modemIsPlugged = getIsModemPlugged();
    m_restartModem = new RestartModem( m_settings );
    
    connect( m_restartModem, SIGNAL(started()), this, SIGNAL(restartModemStarted()) );
    connect( m_restartModem, SIGNAL(finished()), this, SIGNAL(restartModemFinished()) );
    
    
    m_connectThread = new ConnectThread(m_settings, &getModemState);
    
    connect( m_connectThread, SIGNAL(finished()), this, SLOT(connectingFinished()) );
    connect( m_connectThread, SIGNAL(started()), this, SLOT(connectingStarted()) );
    connect( m_connectThread, SIGNAL(synchronizing()), this, SIGNAL(synchronizing()) );
    connect( m_connectThread, SIGNAL(threadMessage(const QString &)), this, SLOT(writeLog(const QString &)) );
    connect( m_connectThread, SIGNAL(debugMsg(QByteArray)), this, SLOT(debugConnecting(QByteArray)) );
    
    
    connect( m_compilerThread, SIGNAL(finished()), this, SLOT(compilationFinished()) );
    connect( m_compilerThread, SIGNAL(started()),  this, SLOT(compilationStarted()) );
  
	initModemsMap();
	
    QListIterator<Modem> it(m_modems);
	while (it.hasNext())
	{
		Modem modem = it.next();
		printf("it: %s\n", modem.name.toStdString().c_str());
		
		if ( modem.usb_id.contains(m_settings->value("vendor_id").toInt(),
			 m_settings->value("product_id").toInt()) )
		{
			printf("oook\n");
			m_compilerThread->setDriverName( modem.driver );
			break;
		}
	}
	

   	int test = m_compilerThread->testDriver();
    	
   	if ( test != 0 )
   	{
   		QEventLoop el;
   		connect( m_compilerThread, SIGNAL(finished()), &el, SLOT(quit()) );
   		
   		writeLog( "UbuDSL_Daemon::UbuDSL_Daemon: modem's driver test failed with code: " + QString::number(test) );
   		
   		m_compilerThread->start();
   		printf("sssssss\n");
   		el.exec();
   		
   		disconnect( m_compilerThread, SIGNAL(finished()), &el, SLOT(quit()) );
	}
    
    if ( m_modemIsPlugged && m_settings->value("connect_at_boot", false).toBool()
    	 && getConnectionState() == ConnectionState::Disconnected )
    {
    	writeLog( "UbuDSL_Daemon(): modem is plugged in, connect at boot." );
    	emit establishConnection();
    }
    
    m_timer->setInterval( 1000 );
    m_deviceTimer->setSingleShot( true );
    connect( m_deviceTimer, SIGNAL(timeout()), this, SLOT(verifyDevice()) );
    
    QDBusConnection::systemBus().connect(QString(), QString(), "org.freedesktop.Hal.Manager", "DeviceAdded", this, SLOT(deviceAdded(QString)));
    QDBusConnection::systemBus().connect(QString(), QString(), "org.freedesktop.Hal.Manager", "DeviceRemoved", this, SLOT(deviceRemoved(QString)));
    
    connect( m_timer, SIGNAL(timeout()), this, SLOT(getTransmissionStatistics()) );
    connect( this, SIGNAL(connectionEstablished()), this, SLOT(verifyDNS()) );
    connect( this, SIGNAL(connectionEstablished()), m_timer, SLOT(start()) );
    connect( this, SIGNAL(connectionTerminated()), m_timer, SLOT(stop()) );


    m_connectedRegExp.setPattern("^local\\s+IP\\s+address\\s+[0-9]{1,4}\\.[0-9]{1,4}\\.[0-9]{1,4}\\.[0-9]{1,4}");
    m_connectingRegExp.setPattern("^Connect:\\s+[a-z]{2,4}[0-9}{1}\\s+<-->\\s+[a-z]{2,4}[0-9}{1}");
    m_invalidDataRegExp.setPattern("^(rcvd \\[CHAP Failure id=.{1,5} \"permission denied\"\\])|(CHAP authentication failed: permission denied)|(sent \\[LCP TermReq id=.{1,5} \"Failed to authenticate ourselves to peer\"\\])");
    
    getModemState();
    
    if ( getConnectionState() == ConnectionState::Connected )
    {
    	writeConnectionLog("Registered IP: " + getIP());
    	getConnectionTime();
    	m_timer->start();
    }
}

UbuDSL_Daemon::~UbuDSL_Daemon()
{
}

void UbuDSL_Daemon::initModemsMap()
{
	if ( debug )
		printf( "UbuDSL_Daemon::initModemsMap(): begin...\n" );
	
	Modem sagem_fast_800_e4;
	
	sagem_fast_800_e4.name = "Sagem F@st 800 E4";
	sagem_fast_800_e4.driver = "ueagle-atm";
	sagem_fast_800_e4.usb_id.insert( 0x1110, 0x9041 );
	sagem_fast_800_e4.usb_id.insert( 0x1110, 0x9042 );
	
	
	Modem sagem_fast_800;
	
	sagem_fast_800.name = "Sagem F@st 800";
	sagem_fast_800.driver = "ueagle-atm";
	sagem_fast_800.usb_id.insert( 0x1110, 0x9010 );
	sagem_fast_800.usb_id.insert( 0x1110, 0x900f );
	sagem_fast_800.usb_id.insert( 0x1110, 0x9021 );
	sagem_fast_800.usb_id.insert( 0x1110, 0x9022 );
	sagem_fast_800.usb_id.insert( 0x1110, 0x9023 );
	sagem_fast_800.usb_id.insert( 0x1110, 0x9024 );
	sagem_fast_800.usb_id.insert( 0x1110, 0x9031 );
	sagem_fast_800.usb_id.insert( 0x1110, 0x9032 );
	
	
	Modem thomson_speedtouch_330;
	
	thomson_speedtouch_330.name = "Thomson Speedtouch 330";
	thomson_speedtouch_330.driver = "speedtch";
	thomson_speedtouch_330.usb_id.insert( 0x06b9, 0x4061 );
	
	
	Modem zte_zxdsl_852;
	
	zte_zxdsl_852.name = "ZTE ZXDSL 852";
	zte_zxdsl_852.driver = "unicorn_usb_eth";
	zte_zxdsl_852.usb_id.insert( 0x0483, 0x0138 );
	
	
	Modem cxacru_based;
	
	cxacru_based.name = "CXACRU modem";
	cxacru_based.driver = "cxacru";
	cxacru_based.usb_id.insert( 0x0572, 0xcafe );
	cxacru_based.usb_id.insert( 0x0572, 0xcb00 );
	cxacru_based.usb_id.insert( 0x0572, 0xcb01 );
	cxacru_based.usb_id.insert( 0x0572, 0xcb02 );
	cxacru_based.usb_id.insert( 0x0572, 0xcb06 );
	cxacru_based.usb_id.insert( 0x0572, 0xcb07 );
	cxacru_based.usb_id.insert( 0x08e3, 0x0100 );
	cxacru_based.usb_id.insert( 0x08e3, 0x0102 );
	cxacru_based.usb_id.insert( 0x0eb0, 0x3457 );
	cxacru_based.usb_id.insert( 0x1803, 0x5510 );
	cxacru_based.usb_id.insert( 0x0675, 0x0200 );
	cxacru_based.usb_id.insert( 0x0586, 0x330a );
	cxacru_based.usb_id.insert( 0x0586, 0x330b );
	cxacru_based.usb_id.insert( 0x0659, 0x0020 );
	cxacru_based.usb_id.insert( 0x0509, 0x0812 );
	cxacru_based.usb_id.insert( 0x100d, 0xcb01 );
	cxacru_based.usb_id.insert( 0x100d, 0x3342 );

	
	m_modems.append( sagem_fast_800_e4 );
	m_modems.append( sagem_fast_800 );
	m_modems.append( thomson_speedtouch_330 );
	m_modems.append( zte_zxdsl_852 );
	m_modems.append( cxacru_based );
	
	if ( debug )
		printf( "UbuDSL_Daemon::initModemsMap(): end...\n" );
}
void UbuDSL_Daemon::writeLog( const QString &str )
{
	if ( debug )
		printf( "%s\n", qPrintable( str ) );
	
	m_mutexLog.lock();
	
    if (m_logFile->open(QIODevice::WriteOnly|QIODevice::Append))
    {
        QByteArray dateArray;
        dateArray.append(QDateTime::currentDateTime().toString("dd-MM-yyyy hh:mm:ss "));
        m_logFile->write(dateArray);
        m_logFile->write(str.toUtf8());
        m_logFile->write("\n");
        m_logFile->close();
    }
    
    m_mutexLog.unlock();
}


void UbuDSL_Daemon::debugHandler( QtMsgType type, const char *msg )
{
	if ( type == QtFatalMsg )
		printf( "Fatal: " );
	else
		printf( "Debug: " );
	
	printf( "%s\n", msg );
	
	m_mutexLog.lock();
	
    if (m_logFile->open(QIODevice::WriteOnly|QIODevice::Append))
    {
        QByteArray dateArray;
        dateArray.append(QDateTime::currentDateTime().toString("dd-MM-yyyy hh:mm:ss "));
          
        m_logFile->write(dateArray);
        
        if ( type == QtFatalMsg )
        	m_logFile->write( "UbuDSL_Daemon::debugHandler (fatal): ");
        else
        	m_logFile->write( "UbuDSL_Daemon::debugHandler: ");
        
        m_logFile->write(msg);
        m_logFile->write("\n");
        m_logFile->close();
        
        if ( type == QtFatalMsg )
        	abort();
    }
    
    m_mutexLog.unlock();
}


// Read the ppp interface name
QString UbuDSL_Daemon::getInterface()
{
    QFile pidFile("/var/run/ppp-ubudsl.pid");

    if (pidFile.exists() && pidFile.open(QFile::ReadOnly))
    {
    	QRegExp ifaceRegExp("(ppp[0-9]{1})");
        
    	while (!pidFile.atEnd())
    	{
    		QString line = pidFile.readLine(10);
    		
    		if (line.contains(ifaceRegExp))
    		{
    			pidFile.close();
    			return ifaceRegExp.cap(1);
    		}
    	}
    	
    	pidFile.close();
    }

    QDir dir("/var/run");
    
    QRegExp pidRegExp("^(ppp[0-9]{1})\\.pid");
    QStringList ppps = dir.entryList(QDir::NoSymLinks|QDir::Files);
    
    foreach ( QString pid, ppps )
    {
    	if ( pid.contains(pidRegExp) )
    		return pidRegExp.cap(1);
    }
    
	if ( debug )
		printf( "UbuDSL_Daemon::getInterface(): returning default (ppp0)\n" );
	
    return "ppp0";
}

const QString UbuDSL_Daemon::getIP(QString interface)
{
	if (interface.isEmpty())
		interface = getInterface();
	
	QString ip;
	
    QNetworkInterface *networkInterface = new QNetworkInterface( QNetworkInterface::interfaceFromName(interface) );
    if ( networkInterface->isValid() )
    {
        QList<QNetworkAddressEntry> entry = networkInterface->addressEntries();
        ip = entry.at(0).ip().toString();
    }
    else
    {
    	writeLog(QString("UbuDSL_Daemon::getIP(): interface %1 is invalid!").arg(interface));
        ip = "...";
    }

    delete networkInterface;
    
	if ( debug )
		printf( "UbuDSL_Daemon::getIP(): returning IP: %s\n", qPrintable( ip ) );
    
    return ip;
}

const QString UbuDSL_Daemon::getDefaultRoute()
{
    QFile routeFile("/proc/net/route");
    QString route;

    if  (routeFile.open(QIODevice::ReadOnly))
    {
        QString routeData( routeFile.readAll().data() );
        routeData.replace( QRegExp("[ ]{1,10}"), "\n" );
        routeData.replace( QRegExp("[\t]{1,10}"), "\n" );
        QStringList routeEntries = routeData.split( "\n" );

        foreach ( QString entry, routeEntries )
        {
            QRegExp regExp( "[a-z]{3,4}[0-9]{1}" );// e.x. ppp1
            
            if ( ( regExp.indexIn( entry ) > -1 ) )
            {
                route = entry;// We don't do break because the last matching
                              // entry contains the default route interface
            }
        }
        
        routeFile.close();
    }

    if ( route.isEmpty() )
        route = "...";

	if ( debug )
		printf( "UbuDSL_Daemon::getIP(): returning Default Route: %s\n", qPrintable( route ) );
	
    return route;
}

void UbuDSL_Daemon::setConnectAtBoot(bool connect)
{
    m_settings->setValue( "connect_at_boot", connect );
}

void UbuDSL_Daemon::setConnectAtModemPlug(bool connect)
{
    m_settings->setValue( "connect_at_modem_plug", connect );
}

bool UbuDSL_Daemon::readUnsignedFromFile( const QString &fileName, unsigned int &value )
{
    FILE* file = fopen( fileName.toLatin1().constData(), "r" );
    if ( file != NULL )
    {
        if ( fscanf( file, "%u", &value ) > 0 )
        {
            fclose( file );
            return true;
        }
        fclose( file );
    }

    return false;
}

bool UbuDSL_Daemon::readLongFromFile( const QString &fileName, unsigned long &value )
{
    FILE* file = fopen( fileName.toLatin1().constData(), "r" );
    if ( file != NULL )
    {
        if ( fscanf( file, "%lu", &value ) > 0 )
        {
            fclose( file );
            return true;
        }
        fclose( file );
    }

    return false;
}

QStringList UbuDSL_Daemon::getLoginAndPassword()
{
    QFile papSecrets("/etc/ppp/pap-secrets");

    if ( papSecrets.open(QFile::ReadOnly) )
    {
    	QRegExp secretRegExp("\"(.{1,50})\"\\s+\\*\\s+\"(.{1,50})\"\\s+\\*");
        while (!papSecrets.atEnd())
        {
        	QString line = papSecrets.readLine(100);
        	if (line.contains(secretRegExp))
        	{
        		papSecrets.close();
        		return QStringList() << secretRegExp.cap(1) << secretRegExp.cap(2);
        	}
        }
    }
    else
    	writeLog("UbuDSL_Daemon::getLoginAndPassword(): could not open pap-secrets file!");

    return QStringList();
}

void UbuDSL_Daemon::getTransmissionStatistics(QString interface)
{
	//QTime time(0,0,0,0);
	//time.start();
	if (interface.isEmpty())
		interface = getInterface();
	
	UbuDSL::Stats stats;
	stats.rx_bytes  = 0;
	stats.rx_errors = 0;
	stats.tx_bytes  = 0;
	stats.tx_errors = 0;
	stats.rx_bps    = 0;
	stats.tx_bps    = 0;
	
	if( !readLongFromFile("/sys/class/net/" + interface + "/statistics/rx_bytes", stats.rx_bytes)
	    && debug )
	{
		printf( "UbuDSL_Daemon::getTransmissionStatistics(): could not get rx_bytes for interface %s\n",
				qPrintable( interface ) );
	}
	
	if( !readLongFromFile("/sys/class/net/" + interface + "/statistics/tx_bytes", stats.tx_bytes)
		&& debug )
	{
		printf( "UbuDSL_Daemon::getTransmissionStatistics(): could not get tx_bytes for interface %s\n",
				qPrintable( interface ) );
	}
	
	if( !readUnsignedFromFile("/sys/class/net/" + interface + "/statistics/rx_errors", stats.rx_errors)
		&& debug )
	{
		printf( "UbuDSL_Daemon::getTransmissionStatistics(): could not get rx_errors for interface %s\n",
				qPrintable( interface ) );
	}
	
	if( !readUnsignedFromFile("/sys/class/net/" + interface + "/statistics/tx_errors", stats.tx_errors)
		&& debug )
	{
		printf( "UbuDSL_Daemon::getTransmissionStatistics(): could not get tx_errors for interface %s\n",
				qPrintable( interface ) );
	}
	
	if (stats.rx_bytes > m_rx_bytes)
	{
		m_rx_bytes = stats.rx_bytes;
	}
	else if (stats.rx_bytes < m_rx_bytes)
	{
		writeConnectionLog( QString( "RX bytes: %1" ).arg( m_rx_bytes ) );
		m_rx_bytes = stats.rx_bytes;
	}
	
	if (stats.tx_bytes > m_tx_bytes)
	{
		m_tx_bytes = stats.tx_bytes;
	}
	else if (stats.tx_bytes < m_tx_bytes)
	{
		writeConnectionLog(QString("TX bytes: %1").arg(m_tx_bytes));
		m_tx_bytes = stats.tx_bytes;
	}
		
	static unsigned int old_rx_bytes = stats.rx_bytes;
	static unsigned int old_tx_bytes = stats.tx_bytes;
	
	stats.rx_bps = stats.rx_bytes - old_rx_bytes;
	stats.tx_bps = stats.tx_bytes - old_tx_bytes;
	
	old_rx_bytes = stats.rx_bytes;
	old_tx_bytes = stats.tx_bytes;

	emit transmissionSpeed ( stats.rx_bps,    stats.tx_bps    );
	emit transmissionErrors( stats.rx_errors, stats.tx_errors );
	emit transmissionBytes ( stats.rx_bytes,  stats.tx_bytes  );
	
	if ( m_connectionTime.isValid() && !m_connectionTime.isNull() )
	{
		m_connectionTime = m_connectionTime.addSecs(1);
		emit connectionTime( m_connectionTime.hour(),
				             m_connectionTime.minute(),
				             m_connectionTime.second() );
	}
}

void UbuDSL_Daemon::writeConnectionLog(const QString &str)
{
	QDir logDir("/var/log/ubudsl");
	
	if ( !logDir.exists() && !logDir.mkpath( "/var/log/ubudsl" ) )
		return;
	
	QString today = QDate::currentDate().toString( "dd-MM-yyyy" );
	
	QFile connectionLog( "/var/log/ubudsl/" + today + ".log" );
	
	if ( connectionLog.open(QFile::WriteOnly|QFile::Append) )
	{
		connectionLog.write( qPrintable(QDateTime::currentDateTime().toString("dd.MM.yyyy hh:mm:ss: ")) );
		connectionLog.write( qPrintable(str) );
		connectionLog.write( "\n" );
		
		connectionLog.close();
	}
}

void UbuDSL_Daemon::establishConnection()
{
	if ( m_connectThread->isRunning() )
	{
		writeLog( "Can not establish connection, thread is still running!" );
		return;
	}
	
	if ( getConnectionState() == ConnectionState::Connected )
	{
		if ( debug )
			printf( "UbuDSL_Daemon::establishConnection(): already connected...\n" );
	}
	
	switch (getModemState())
	{
		case Synchronizing:
			printf( "Modem is synchronizing, returning...\n" );
			break;
		case Unknown:
			printf( "Modem state is unknown, returning...\n" );
			break;
		case SynchronizationFailed:
			printf( "Modem synchronization failed, returning...\n" );
			break;
		case Error:
			printf( "Modem driver returned error, returning...\n" );
			break;
	}
	
	m_connecting = true;
	m_connectThread->start();
}

void UbuDSL_Daemon::deviceRemoved( const QString udi )
{
	m_settings->sync();
	
	if ( udi == m_settings->value("udi").toString() )
	{
		if ( debug )
			printf( "UbuDSL_Daemon::deviceRemoved: modem removed\n" );
		
		m_modemIsPlugged = false;
		terminateConnection();
		emit modemUnplugged();
	}
	else if ( udi == m_newUDI )
	{
		if ( debug )
			printf( "UbuDSL_Daemon::deviceRemoved: new modem removed\n" );
		
		emit newModemUnplugged();
		m_newUDI = "#";
	}
}

void UbuDSL_Daemon::deviceAdded( const QString udi )
{
	QDBusInterface device( "org.freedesktop.Hal", udi,
                           "org.freedesktop.Hal.Device",
                           QDBusConnection::systemBus() );

	
	QDBusReply<int> vendor_id  = device.call ( "GetProperty", "usb_device.vendor_id" );
	QDBusReply<int> product_id = device.call ( "GetProperty", "usb_device.product_id" );

    if ( vendor_id.isValid() && product_id.isValid() )
    {
    	qDebug() << "device added:" << udi;
    	m_lastUDI = udi;
    	m_deviceTimer->stop();
    	m_deviceTimer->start(5000);
    }
}

void UbuDSL_Daemon::verifyDevice()
{
	m_settings->sync();
	
	m_newUDI = m_lastUDI;
	
	if ( debug )
		printf( "UbuDSL_Daemon::verifyDevice: %s\n", qPrintable( m_newUDI ) );

	QDBusInterface device( "org.freedesktop.Hal", m_newUDI,
                           "org.freedesktop.Hal.Device",
                           QDBusConnection::systemBus() );

	
	QDBusReply<int> vendor_id  = device.call ( "GetProperty", "usb_device.vendor_id" );
	QDBusReply<int> product_id = device.call ( "GetProperty", "usb_device.product_id" );
	
	if ( !vendor_id.isValid() && !product_id.isValid() )
	{
		m_newUDI = "#";
		return;
	}
	
	if ( vendor_id.value()  == m_settings->value("vendor_id", -1).toInt() &&
		 product_id.value() == m_settings->value("product_id", -1).toInt() )
	{
		m_modemIsPlugged = true;
		
		if ( debug )
			printf( "UbuDSL_Daemon::verifyDevice: modem plugged\n" );
		
		emit modemPlugged(m_settings->value("modem", "").toString());
		
		if ( getConnectionState() == ConnectionState::Disconnected &&
			 m_settings->value("autoconnect", false).toBool() )
		{
			if ( debug )
				printf( "UbuDSL_Daemon::verifyDevice: emit establishConnection()\n" );
			
			emit establishConnection();
		}
		
		return;
	}

	QListIterator<Modem> it(m_modems);
	while (it.hasNext())
	{
		Modem modem = it.next();
		
		if ( !m_modemIsPlugged && modem.usb_id.contains(vendor_id.value(), product_id.value()) )
		{
			if ( debug )
				printf( "UbuDSL_Daemon::verifyDevice: new modem plugged.\n" );
			
			m_newVendorID  = vendor_id.value();
			m_newProductID = product_id.value();
			m_newModemName = modem.name;
			
			emit newModemPlugged(m_newModemName);
			return;
		}
	}

	m_newUDI = "#";
}

bool UbuDSL_Daemon::getIsModemPlugged()
{
	QDBusReply<QStringList> reply = m_halManager->call( "GetAllDevices", "udi" );
	
	if ( !reply.isValid() && m_justRun )
	{
		QDBusError error = m_halManager->lastError();

		writeLog( "UbuDSL_Daemon::getIsModemPlugged(): last error of m_halManager(1): [" + error.name() + ": " + error.message() + "]." );
		writeLog( "UbuDSL_Daemon::getIsModemPlugged(): waiting 3s and retrying to get reply..." );

		QEventLoop eventLoop;
		QTimer::singleShot( 3000, &eventLoop, SLOT(quit()) );
		eventLoop.exec();

		delete m_halManager;
		m_halManager = new QDBusInterface("org.freedesktop.Hal", "/org/freedesktop/Hal/Manager", "org.freedesktop.Hal.Manager", QDBusConnection::systemBus());
		reply = m_halManager->call( "GetAllDevices", "udi" );
	}

	m_justRun = false;

	if ( !reply.isValid() )
	{
		QDBusError error = m_halManager->lastError();
		writeLog( "UbuDSL_Daemon::getIsModemPlugged(): last error of m_halManager(2): [" + error.name() + ": " + error.message() + "]." );
		writeLog( "UbuDSL_Daemon::getIsModemPlugged(): invalid dbus reply, returning false!" );
		return false;
	}
	
	return reply.value().contains(m_settings->value("udi", "#").toString());
}

void UbuDSL_Daemon::verifyDNS()
{
// 	if ( !dns.isFileCorrect() )
// 	{
// 		emit incorrectDNS();
// 		setOpenDNS();
// 	}
    QStringList dnsServers = m_settings->value("DNS", QStringList()).toStringList();

    if ( !dns.setServers(dnsServers) )
	setOpenDNS();
}

void UbuDSL_Daemon::setOpenDNS()
{
	if ( dns.setServers( QStringList() << "208.67.222.222" << "208.67.220.220" ) )
		emit settingOpenDNS(true);
	else
		emit settingOpenDNS(false);
}

int UbuDSL_Daemon::getModemState()
{
	int vendor_id = m_settings->value("vendor_id", 0).toInt();
	
	QRegExp booting("#####");
	QRegExp synchronizing("#####");
	QRegExp synchronized("#####");
	QRegExp synchronizationFailed("#####");
	QRegExp error("#####");
	
	QRegExp startingString("####");
	
	if ( vendor_id == 0x1110 ) //ueagle-atm
	{
		startingString.setPattern("\\[ueagle-atm\\]");
		booting.setPattern("(\\[ueagle-atm\\] \\(re\\)booting started)|(\\[ueagle-atm\\] Modem started, waiting synchronization)");
		synchronized.setPattern("\\[ueagle-atm\\] modem operational");
	}
	else if ( vendor_id == 0x0483 ) //unicorn_usb_eth
	{
		startingString.setPattern("unicorn_msw:");
		
		booting.setPattern("(unicorn_msw: Current Modem State \\(\\d\\): ACTIVATING)|"
				           "(unicorn_msw: Event Reported \\(\\d\\): Initializing)|"
				           "(unicorn_msw: Current Modem State \\(\\d\\): INITIALIZING)|"
				           "(unicorn_msw: Current Modem State \\(\\d\\): Ghs_HANDSHAKING)");
		
		synchronized.setPattern("(unicorn_msw: Current Modem State \\(\\d\\): SHOWTIME_L\\d)|"
				                "(unicorn_msw: Event Reported \\(\\d\\): Showtime)");
	}
	else if ( vendor_id == 0x06b9 ) //speedtch
	{
		startingString.setPattern("(ATM dev \\d:)|(speedtch \\d-\\d:\\d.\\d:)");
		synchronized.setPattern("ATM dev \\d: ADSL line is up \\(([0-9]{1,5}) kb/s down | ([0-9]{1,5}) kb/s up\\)");
		synchronizing.setPattern("ATM dev \\d: ADSL line is synchronising");
		error.setPattern("speedtch \\d-\\d:\\d.\\d: speedtch_heavy_init: firmware upload failed");
	}
	else //cxacru
	{
		startingString.setPattern("ATM dev \\d:");
		synchronized.setPattern("ATM dev \\d: ADSL line: up");
		booting.setPattern("ATM dev \\d: ADSL line: initializing");
		synchronizing.setPattern("ATM dev \\d: ADSL line: attempting to activate");
	}
	
	const int len = 100000;
	char *buf = new char[len];
	
	if ( klogctl(3, buf, len) <= 0 )
	{
		writeLog("UbuDSL_Daemon::getModemState(): klogctl() <= 0!");
		return Unknown;
	}
	
	QString log(buf);
	delete buf;
	
	int lastPos = log.count();
	
	for (int i=0; i<5; i++)
	{
		qDebug() << "i:" << i;
		int idx = log.lastIndexOf(startingString, lastPos);
		
		if ( idx < 0 )
		{
			if ( debug )
				printf( "UbuDSL_Daemon::getModemState(): idx < 0\n" );
			
			return Unknown;
		}
		
		lastPos = idx;
		
		if ( log.indexOf(synchronized, lastPos) == idx )
		{
			if ( debug )
				printf( "UbuDSL_Daemon::getModemState(): synchronized.\n" );

			return Synchronized;
		}
		else if ( log.indexOf(synchronizing, lastPos) == idx )
		{
			if ( debug )
				printf( "UbuDSL_Daemon::getModemState(): synchronizing.\n" );

			return Synchronizing;
		}
		else if ( log.indexOf(booting, lastPos) == idx )
		{
			if ( debug )
				printf( "UbuDSL_Daemon::getModemState(): booting\n" );
			
			return Booting;
		}
		else if ( log.indexOf(synchronizationFailed, lastPos) == idx )
		{
			if ( debug )
				printf( "UbuDSL_Daemon::getModemState(): synchronization failed\n" );
			
			return SynchronizationFailed;
		}
		else if ( log.indexOf(error, lastPos) == idx )
		{
			if ( debug )
				printf( "UbuDSL_Daemon::getModemState(): error\n" );
			
			return Error;
		}
	}
	
	return Unknown;
}

void UbuDSL_Daemon::removePidFiles( const QString &killedIface )
{
	QFile::remove( "/var/run/" + killedIface + ".pid" );
	QFile::remove( "/var/run/ppp-ubudsl.pid" );
}

int UbuDSL_Daemon::getConnectionState()
{
	if ( m_connecting )
		return ConnectionState::Connecting;
	
	QNetworkInterface interface = QNetworkInterface::interfaceFromName(getInterface());
	if (interface.isValid() && !interface.addressEntries().at(0).ip().toString().isEmpty() )
		return ConnectionState::Connected;
	
	return ConnectionState::Disconnected;
}

QTime UbuDSL_Daemon::getConnectionTime()
{
    unsigned int pid = 0;

    if ( readUnsignedFromFile("/var/run/ppp-ubudsl.pid", pid) ||
    	 readUnsignedFromFile("/var/run/" + getInterface() + ".pid", pid) )
    {
    	QRegExp timeRegExp("\\s*([0-9]{2})?:?([0-9]{2}):([0-9]{2})");
        QString pppTime;
        
        QProcess getTimeProcess;
        getTimeProcess.start( "ps", QStringList() << "-p" << QString::number(pid) << "-o" << "etime=" );
        getTimeProcess.waitForFinished(2000);

        pppTime = getTimeProcess.readLine(50);

        if ( pppTime.contains(timeRegExp) )
        {
        	m_connectionTime = QTime( timeRegExp.cap(1).toInt(),
       				    			  timeRegExp.cap(2).toInt(),
       				    			  timeRegExp.cap(3).toInt() );
       		
       		return m_connectionTime;
        }
    }

	if ( debug )
		printf( "UbuDSL_Daemon::getConnectionTime(): returning QTime()!\n" );
    
    QTimer::singleShot( 1000, this, SLOT(getConnectionTime()) );
    m_connectionTime = QTime();
    
    return m_connectionTime;
}

void UbuDSL_Daemon::debugConnecting(QByteArray msg)
{
	writeLog( "UbuDSL_Daemon::debugConnecting(): " + msg );

	if ( QString(msg).contains(m_connectingRegExp) )
	{
		qDebug() << "connectingRegExp";
		m_connecting = true;
		m_timer->stop();
	}
	else if ( QString(msg).contains(m_connectedRegExp) )
	{
		emit connectionEstablished();
		emit IP( getIP() );
		emit defaultRoute( getDefaultRoute() );
		
		m_connecting = false;
		getConnectionTime();
		m_timer->start();
	}
	else if ( QString(msg).contains(m_invalidDataRegExp) )
	{
		m_connecting = false;
		terminateConnection(false);
		emit invalidAccountData();
	}
	else if ( msg.contains("Timeout waiting for PADO packets") )
	{
		terminateConnection(false);
		m_connecting = false;
		emit connectionTimeout();
	}
}

void UbuDSL_Daemon::connectingStarted()
{
	m_connecting = true;
	emit setEnabledConnect(false);
	emit establishingConnection();
}

void UbuDSL_Daemon::connectingFinished()
{
	m_connecting = false;
	emit setEnabledConnect(true);
	
	switch ( m_connectThread->getReturnState() )
	{
		case ReturnState::Timeout:
			emit connectionTimeout();
			break;
		case ReturnState::SynchronizationFailed:
			emit synchronizationFailed();
			break;
	}

	emit connectionState(getConnectionState());
}

void UbuDSL_Daemon::terminateConnection( bool message )
{
	qDebug() << 200;
	if ( debug )
		printf( "UbuDSL_Daemon::terminateConnection()\n" );
	
	QProcess pkill;
	unsigned int pid = 0;
	int ok = -1;
	qDebug() << 201;
	QString ifaceToKill = getInterface();
	qDebug() << 202;
	if ( readUnsignedFromFile("/var/run/ppp-ubudsl.pid", pid) || 
		 readUnsignedFromFile("/var/run/" + ifaceToKill + ".pid", pid) )
	{
		qDebug() << 203;
		qDebug() << "pid:" << pid;
		( kill(pid, 9) == 0 ) ? ok = true : ok = false;
		qDebug() << 205;
	}
	qDebug() << 206;
	if ( readUnsignedFromFile("/var/run/nas0.pid", pid) )
	{
		qDebug() << 207;
		pkill.start("kill", QStringList() << "-9" << QString(pid) );
		pkill.waitForFinished(500);
		qDebug() << 208;
		if ( ok == 0 )
			ok = pkill.exitCode();
	}
	
	if ( ok != 0 )
	{
		qDebug() << 209;
		pkill.start("killall", QStringList() << "-9" << "pppd" << "br2684ctl_amd64" << "br2684ctl_i386" );
		pkill.waitForFinished(500);
	}
	qDebug() << 210;
	removePidFiles( ifaceToKill );
	qDebug() << 211;
        if ( message )
	  emit connectionTerminated();
	qDebug() << 212;
	
	m_timer->stop();
}

void UbuDSL_Daemon::restartModem()
{
	if ( m_restartModem->isRunning() )
	{
		writeLog( "UbuDSL_Daemon::restartModem(): Can not restart modem, thread is still running!" );
		return;
	}
	
	if ( getConnectionState() != ConnectionState::Disconnected )
		terminateConnection();
	
	m_restartModem->start();
}

int UbuDSL_Daemon::getIdlenessTime()
{
	return m_settings->value("idleness", 0).toInt();
}

void UbuDSL_Daemon::setIdlenessTime(const int &idleness)
{
	return m_settings->setValue("idleness", idleness);
}

void UbuDSL_Daemon::setAccountData( const QString &login, const QString &password )
{
	bool ok = true;
    QFile file;
    QStringList secretFiles;
    secretFiles << "/etc/ppp/pap-secrets" << "/etc/ppp/chap-secrets";
    

    QRegExp secretRegExp("^\"(.{1,50})\"\\s+\\*\\s+\"(.{1,50})\"\\s+\\*");
    QString secretEntry = "\"" + login + "\" * \"" + password + "\" *";
    
    foreach (QString fName, secretFiles )
    {
    	file.setFileName(fName);
	    if ( file.open(QFile::ReadOnly) )
	    {
	    	QString data = file.readAll();
	    	file.close();
	    	
	    	if ( data.contains(secretRegExp) )
	    		data = data.replace(secretRegExp, secretEntry);
	    	else
	    		data = data.append(secretEntry) + "\n";
	    	
	    	if (file.open(QFile::WriteOnly|QFile::Truncate))
	    	{
	    		file.write(qPrintable(data));
	    		file.close();
	    	}
	    	else
	    	{
	    		ok = false;
	    		writeLog("UbuDSL_Daemon::setAccountData(): could not open"
	    				 "secret file in write only mode!");	    		
	    	}
	    }
	    else
	    {
	    	ok = false;
    		writeLog("UbuDSL_Daemon::setAccountData(): could not open"
    				 "secret file in read only mode!");
	    }
	}
    
    QStringList peerFiles;
    peerFiles << "/etc/ppp/peers/ubudsl" << "/etc/ppp/peers/ubudsl_unicorn";
    
    QRegExp userRegExp("user \".{1,50}\"");
    QString userEntry = "user \"" + login + "\"";
    
    foreach (QString fName, peerFiles )
    {
    	file.setFileName(fName);
	    if ( file.open(QFile::ReadOnly) )
	    {
	    	QString data = file.readAll();
	    	file.close();
	    	
	    	if ( data.contains(userRegExp) )
	    		data = data.replace(userRegExp, userEntry);
	    	else
	    		data = data.append(userEntry) + "\n";
	    	
	    	if (file.open(QFile::WriteOnly|QFile::Truncate))
	    	{
	    		file.write(qPrintable(data));
	    		file.close();
	    	}
	    	else
	    	{
	    		ok = false;
	    		writeLog("UbuDSL_Daemon::setAccountData(): could not open "
	    				 "peer file in write only mode!");
	    	}
	    }
	    else
	    {
	    	ok = false;
    		writeLog("UbuDSL_Daemon::setAccountData(): could not open "
    				 "peer file in read only mode!");
	    }
	}
    
    emit accountDataChanged(ok);
}

void UbuDSL_Daemon::useNewModem()
{
	emit modemPlugged(m_newModemName);
	
	m_settings->setValue("modem", m_newModemName);
	m_settings->setValue("vendor_id", m_newVendorID);
	m_settings->setValue("product_id", m_newProductID);
	m_settings->setValue("udi", m_newUDI);
	
	m_newModemName = "";
	m_newVendorID  = 0;
	m_newProductID = 0;
	m_newUDI       = "#";
	
	if ( m_settings->value("connect_at_modem_plug", false).toBool() )
		establishConnection();
}

void UbuDSL_Daemon::compilationStarted()
{
	m_compiling = true;
	emit compilingDriver();
}

void UbuDSL_Daemon::compilationFinished()
{
	m_compiling = false;
	
	switch ( m_compilerThread->getResult() )
	{
		case CompilerThread::Success:
			emit compilationFinished( true );
			break;
		case CompilerThread::CompilationFailed:
		case CompilerThread::InstallationFailed:
			emit compilationFinished( false );
			break;
	}
}

bool UbuDSL_Daemon::getIsCompiling()
{
	return m_compiling;
}

