#include <QtCore>
#include <QCoreApplication>
#include <unistd.h>

#include "UbuDSL_Daemon.h"
#include "libubudsl/FileLocker.h"
#include "libubudsl/Globals.h"
#include <signal.h>

extern bool  debug;
extern bool  debugToFile;
extern FILE *pFile;

QMutex mutex;

//bool debug = false;
void signalHandler( int i );
//void debugHandler ( QtMsgType type, const char *msg );

int main( int argc, char *argv[] )
{
	qInstallMsgHandler ( UbuDSL_Daemon::debugHandler );
	setDebug( true, "/opt/ubudsl/daemon_verbose.log" );
	
	pid_t pid = fork();

	if ( pid == 0 )
	{
		int FD = lockFile( "/var/lock/ubudsldaemon" );
		if( FD < 0 )
		{
			fprintf( stderr, "UbuDSL Daemon: It seems that the daemon is already running.\n" );
			
			return -2;
		}

	    QCoreApplication app( argc, argv );
		
	    signal( SIGINT,  signalHandler );
	    signal( SIGHUP,  signalHandler );
	    signal( SIGTERM, signalHandler );
	    
	    QStringList args = app.arguments();
	    
	    if ( args.contains( "--debug" )   || args.contains( "-debug" ) || args.contains( "-d" ) ||
	    	 args.contains( "--verbose" ) || args.contains( "-debug" ) || args.contains( "-v" ) )
	    {
	    	debug = true;
	    }
	    
	    UbuDSL_Daemon daemon;
	    Q_UNUSED( daemon );
	    
	    if ( debug )
	    	fprintf( stdout, "UbuDSL Daemon: started.\n" );
	    
	    return app.exec();
	}
	else if ( pid < 0 )
	{
		fprintf( stderr, "UbuDSL Daemon: Failed to fork (%i)!\n", pid );
		return -1;
	}
}

void signalHandler( int i )
{
	printf( "UbuDSL Daemon: Signal %d handled\n", i );
	exit( -1 );
}

/*void debugHandler ( QtMsgType type, const char *msg )
{
    mutex.lock();

    fprintf( pFile, "%s\n", msg );

    if ( type == QtFatalMsg && debugToFile )
    {
    	fprintf( pFile,  "Fatal: %s\n", msg );
        fprintf( stderr, "Fatal: %s\n", msg );
        
        abort();
    }
    else if ( type == QtFatalMsg && !debugToFile )
    {
    	fprintf( pFile, "Fatal: %s\n", msg );
    	
    	abort();
    }
    else
    {
    	fprintf( pFile, "%s\n", msg );
    }

    QFile f("/opt/ubudsl/tmp.log");
    if ( f.open(QFile::WriteOnly|QFile::Append) )
    {
        f.write(msg);
        f.write("\n");
        f.close();
        
        if ( type == QtFatalMsg )
        	abort();
    }
    mutex.unlock();
}*/

